<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use HCM\FPA\ObrigacaoLegal\GRRF\PercorreGRRF;
use HCM\Geral\Controllers\RelatoriosHcmController;


class GRRFController extends Crud {

    protected $GRRFService;
    protected $dataSourceNameGet = 'getGRRF';
    protected $dataSourceNameGetData = 'getCompetencia';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_CDOPERINCLUSAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, RelatoriosHcmController $relatoriosHcmController){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->relatoriosHcmController = $relatoriosHcmController;
        parent::__construct($dataSourceManager);
    }

    public function getGRRF (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if(!is_null($row['DTMESCOMPETENCLIST'])) {
                $row['DTMESCOMPETENCLIST'] = explode(',',$row['DTMESCOMPETENCLIST']);
            }
            
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                if($row['DSFIM']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIM']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
    }
    
    public function setGRRF(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_GRRF";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMGRRF', $row)) {
                    if ($row['NRFORMGRRF'] === '') {
                        $row['NRFORMGRRF'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMGRRF', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMGRRF'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMGRRF', 1, 12, true); 
                }
                
                if(is_array($row['DTMESCOMPETENCLIST'])) {
                    $row['DTMESCOMPETENCLIST'] = implode(',',$row['DTMESCOMPETENCLIST']);
                }
                
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                if(!array_key_exists('DTINICIAL', $row)){
                    $row['DTINICIAL'] = null;
                }
                if(!array_key_exists('DTFINAL', $row)){
                    $row['DTFINAL'] = null;
                }
                // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND") {
                    if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                $row['NRESTRUTRESPONS'] = isset($row['NRESTRUTRESPONS'])?$row['NRESTRUTRESPONS']:null;
                $row['NRHIERARTOMADOR'] = isset($row['NRHIERARTOMADOR'])?$row['NRHIERARTOMADOR']:null;
                $row['NRPESSOARESPONS'] = isset($row['NRPESSOARESPONS'])?$row['NRPESSOARESPONS']:null;
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteGRRF(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_GRRF";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function geraGrrf(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            error_reporting(E_ERROR | E_PARSE);
            
            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];
            $percorreGrrf = new PercorreGRRF($this->entityManager, $this->hcmParameters, $dataSet);
            $resultado = $percorreGrrf->percorreVinculos();
            if( is_string($resultado) ){
                $message = new \Zeedhi\Framework\DTO\Response\Message($resultado, DTO\Response\Message::TYPE_WARNING);
            }elseif($resultado['ARQUIVO'] && !$resultado['ABORT']){
                $row = array('NMARQUIVO' => FPA::getTempDirectory().'/'.strtoupper($resultado['ARQUIVO']));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $row));
                //gera relatório analítico
                $filtro = array('P_MOVIMENTO'=>$dataSet['NRTIPOMOVIMENT'], 'P_NRFORMULARIO'=>$dataSet['NRFORMGRRF'],'P_ESTRUTURARESP'=>$dataSet['NRESTRUTRESPONS'], 'P_COMPETENCIA'=>$this->hcmParameters->getDtCompetencia());
                $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'analitico_grrf', $request, $response);

                if($this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'GRRF'))){
                    $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'GRRF'));
                }else {
                    $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
                }
                
            }elseif ($resultado['ABORT'] === true) {
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'GRRF'));
                if(!$message){
                    $message = 'Processamento não finalizado com erros.<br>';
                    foreach ($resultado['MENSAGENS'] as $mensagem) {
                        if ($mensagem['TIPO'] == \MsgCalculo::ERRO) {
                            $message .= strpos($message, $mensagem['MENSAGEM']) ? null : '<br> - ' . $mensagem['MENSAGEM'];
                        }
                    }
                    $message = new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_ERROR);
                }
            }elseif ($resultado['ERRO'] === false) {
                //gera relatório analítico
                $filtro = array('P_MOVIMENTO'=>$dataSet['NRTIPOMOVIMENT'], 'P_NRFORMULARIO'=>$dataSet['NRFORMGRRF'],'P_ESTRUTURARESP'=>$dataSet['NRESTRUTRESPONS'], 'P_COMPETENCIA'=>$this->hcmParameters->getDtCompetencia());
                $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'analitico_grrf', $request, $response);

                $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
            } else {
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'GRRF'));
            }
            
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            } else {
                $row = array();
                foreach ($message as $mensagem) {
                    $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getCompetencia(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGetData);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        $response->addDataSet(new DataSet($this->dataSourceNameGetData, $rows));
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}