<?php
namespace HCM\FPA\Logic;

use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\HTTP\Kernel;

class EventoUnidadeLogic {
    
    private $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function cadAutoEventoUnidade($row) {
        try {
            foreach($row as $value){
                $existeEvento = false;
                $eventos = $this->verificaEventosUnidade($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtCompetencia(), $value['NRESTRUTURAM']);
                foreach($eventos as $event) {
                    if($event['NREVENTOM'] == $value['NREVENTOM']) {
                        $existeEvento = true;
                    }
                }
                if(!$existeEvento) {
                    $this->insertEventoUnidade($value['NRESTRUTURAM'],$value['NREVENTOM']);
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function insertEventoUnidade($nrestrut, $nrevento){
        try {
            $sql= $this->entityManager->getConnection()->prepare(
                "INSERT INTO FPA_EVENTOUNIDADE (
                    NRORG,
                    NREVENTOUNIDADE,
                    NRESTRUTURAM,
                    NREVENTOM,
                    DTINCLUSAO,
                    DTULTATU,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    CDOPERULTATU) VALUES (
                        :NRORG,
                        :NREVENTOUNIDADE,
                        :NRESTRUTURAM,
                        :NREVENTOM,
                        :DTINCLUSAO,
                        :DTULTATU,
                        :NRORGINCLUSAO,
                        :CDOPERINCLUSAO,
                        :NRORGULTATU,
                        :CDOPERULTATU)"    
            );
    
            $sql->execute(
                array(
                    "NRORG"                 =>  $this->hcmParameters->getNrorgTrabalho(),
                    "NREVENTOUNIDADE"       =>  \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_EVENTOUNIDADE', 1, 12, true),
                    "NRESTRUTURAM"          =>  $nrestrut,
                    "NREVENTOM"             =>  $nrevento,
                    "DTINCLUSAO"            =>  DateUtil::getDataAtual()->format('d/m/Y h:m:s'),
                    "DTULTATU"              =>  DateUtil::getDataAtual()->format('d/m/Y h:m:s'),
                    "NRORGINCLUSAO"         =>  $this->hcmParameters->getNrorgTrabalho(),
                    "CDOPERINCLUSAO"        =>  $this->hcmParameters->getCdoperador(),
                    "NRORGULTATU"           =>  $this->hcmParameters->getNrorgTrabalho(),
                    "CDOPERULTATU"          =>  $this->hcmParameters->getCdoperador()
                )
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaEventosUnidade ($nrorg, $dtmescompetenc, $nrestruturam) {
        
        $sql = "
SELECT 
  EVU.NREVENTOM
FROM 
  FPA_EVENTOUNIDADE EVU
  JOIN FPA_EVENTOM EM
    ON ( EM.NRORG = EVU.NRORG OR EM.NRORG = 0 )
      AND EM.NREVENTOM = EVU.NREVENTOM
  JOIN FPA_EVENTOH EH
    ON ( EH.NRORG = EVU.NRORG OR EM.NRORG = 0 )
      AND EH.NREVENTOM = EVU.NREVENTOM
      AND EH.DTMESCOMPETENC = (SELECT 
                                 MAX(DTMESCOMPETENC)
                               FROM 
                                 FPA_EVENTOH
                               WHERE 
                                 DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                 AND ( NRORG = EVU.NRORG OR NRORG = 0 )
                                 AND NREVENTOM = EM.NREVENTOM )
WHERE 
  EVU.NRORG = :P_NRORG
  AND EVU.NRESTRUTURAM = :P_NRESTRUTURAM
        ";
        
        $params = array(
            'P_NRORG'            => $nrorg,
            'P_DTMESCOMPETENC'   => $dtmescompetenc,
            'P_NRESTRUTURAM'     => $nrestruturam
        );
        
        $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
        
        if (count($result) === 0)
            return null;
        else
            return $result;
    }
    
}
