<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\UploadFileService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class UploadFileController extends Crud {
        
    protected $uploadsFileService;
    protected $dataSourceApagarAnexos = 'gpeAnexos_apagarAnexos';
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, UploadFileService $uploadsFileService){
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
        $this->hcmParameters = $hcmParameters;
        $this->uploadsFileService = $uploadsFileService;
    }
    
    public function saveUploadFile(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if( $row['__is_new'] == false && (count($row['DSPATH']) == 0) ){
                    $update = $this->uploadsFileService->updateNomeGpeAnexos($row);
                    
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('saveUploadFile', array($update)));
                } else {
                    if (!array_key_exists('CDOPERADOR_LOG', $row)) 
                        $row['CDOPERADOR_LOG']= $this->hcmParameters->getCdoperador();
                    $upload = $this->uploadsFileService->setGpeAnexos($row);
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('saveUploadFile', array($upload)));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteUploadFile($request, $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->uploadsFileService->deleteUploadFile($row['DSPATH']);
                $this->dataSourceName = $this->dataSourceApagarAnexos;            
                parent::delete($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function aprovaStatusFile(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $nranexo = $row['NRANEXO'];
            
            $result = $this->uploadsFileService->aprovaStatusFile($nranexo);
            
            $responseResult = array(
                'success' => $result,
            );
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('alteraStatusFile', $responseResult));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}