<?php

namespace HCM\Geral\Controllers;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\VerificacaoService;
use HCM\Util\Messages;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class VerificacaoController extends Crud {

    protected $dataSourceName = 'ger_verificacao';
    protected $dataSourceName_get = 'ger_verificacao_get';
    protected $dataSourceNameParam = 'ger_paramverifica';
    protected $dataSourceNamePerfil = 'ger_perfilverifica';
    protected $dataSourceNamePerfil_get = 'ger_perfilverifica_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, Connection $connection, ParameterBag $parameterBag, HCMParameters $hcmParameters, VerificacaoService $verificacaoService){
        $this->connection = $connection;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->verificacaoService = $verificacaoService;
        parent::__construct($dataSourceManager);
    }

    public function getVerificacao (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName_get);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setVerificacao (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = isset($row['NRORG']) ? $row['NRORG'] : '';
                $nrverificacao = isset($row['NRVERIFICACAO']) ? $row['NRVERIFICACAO'] : '';
                $dsemailverifica = isset($row['DSEMAILVERIFICA']) ? $row['DSEMAILVERIFICA'] : '';
                $dsconsultaverifica = isset($row['DSCONSULTAVERIFICAAUX']) ? $row['DSCONSULTAVERIFICAAUX'] : '';

                // Inserir ou editar
                if ($request->getRoutePath() == '/setVerificacao') {
                    try {
                        $this->connection->beginTransaction();

                        if ($dsemailverifica) {
                            $this->verificacaoService->verificaEmail($dsemailverifica);
                        }

                        $params = $this->verificacaoService->getParametrosConsulta($dsconsultaverifica);
                        if (!$params) {
                            $response->addMessage(new DTO\Response\Message(Messages::HCM00000_GER_VERIF_SAVE_SUCCESS_N_ENCONTR_PARAM, DTO\Response\Message::TYPE_WARNING));
                        } else {
                            if ($row['__is_new'] === true) {
                                if (array_key_exists('NRVERIFICACAO', $row)) {
                                    if ($row['NRVERIFICACAO'] === '') {
                                        $row['NRVERIFICACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GER_VERIFICACAO', 1, 12, true);
                                    }
                                } else {
                                    $row['NRVERIFICACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GER_VERIFICACAO', 1, 12, true);
                                }
                                $this->verificacaoService->novaVerificacao($row['NRVERIFICACAO'], $row['DSVERIFICACAO'], $row['NRTIPOVERIFICA'],
                                    $row['DSEMAILVERIFICA'], $row['MSGVERIFICACAO'], $row['DSCONSULTAVERIFICAAUX']);
                                $nrverificacao = $row['NRVERIFICACAO'];
                            }
                            $this->verificacaoService->atualizaParametros($nrverificacao, $dsconsultaverifica, $nrorg);
                            $response->addMessage(new DTO\Response\Message(Messages::HCM00000_GER_VERIF_SAVE_SUCCESS, DTO\Response\Message::TYPE_MESSAGE));
                            $response->addNotification(new DTO\Response\Notification("1 row(s) persisted with success", DTO\Response\Notification::TYPE_SUCCESS));
                        }

                        $this->connection->commit();
                    } catch (\Exception $e) {
                        $this->connection->rollBack();
                        throw $e;
                    }
                } else {
                    // Ativa a verificação
                    if ($request->getRoutePath() == '/ativarVerificacao') {
                        $this->verificacaoService->ativaVerificacao($nrverificacao, $nrorg);
                        $row['IDATIVO'] = 'S';
                    }

                    // Desativa a verificação
                    if ($request->getRoutePath() == '/desativarVerificacao') {
                        $row['IDATIVO'] = 'N';
                    }

                    parent::save($request, $response);
                }
            }
        } catch(\Exception $e) {
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteVerificacao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = isset($row['NRORG']) ? $row['NRORG'] : '';
                $nrverificacao = isset($row['NRVERIFICACAO']) ? $row['NRVERIFICACAO'] : '';
                $this->verificacaoService->deleteParamsVerificacao($nrorg, $nrverificacao);
                $this->verificacaoService->deletePerfisVerificacao($nrorg, $nrverificacao);
            }
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getParamverifica (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameParam);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setParamverifica (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = isset($row['NRORG']) ? $row['NRORG'] : '';
                $nrverificacao = isset($row['NRVERIFICACAO']) ? $row['NRVERIFICACAO'] : '';
                // @todo validar valor do parâmetro
            }
            $this->dataSourceName = $this->dataSourceNameParam;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getPerfilverifica (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNamePerfil_get);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setPerfilverifica (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = isset($row['NRORG']) ? $row['NRORG'] : '';
                $nrverificacao = isset($row['NRVERIFICACAO']) ? $row['NRVERIFICACAO'] : '';
                $nrperfilacesso = isset($row['NRPERFILACESSO']) ? $row['NRPERFILACESSO'] : '';

                if ($this->verificacaoService->validaVerificacaoPossuiPerfil($nrorg, $nrverificacao, $nrperfilacesso)) {
                    $response->addMessage(new DTO\Response\Message(Messages::HCM00000_GER_PERFILACESSO_ALREADY_EXIST_ERROR, DTO\Response\Message::TYPE_ERROR));
                    return;
                }

                if (array_key_exists('NRPERFILVERIFICA', $row)) {
                    if ($row['NRPERFILVERIFICA'] === '') {
                        $row['NRPERFILVERIFICA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GER_PERFILVERIFICA', 1, 12, true);
                    }
                } else {
                    $row['NRPERFILVERIFICA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GER_PERFILVERIFICA', 1, 12, true);
                }
            }
            $this->dataSourceName = $this->dataSourceNamePerfil;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deletePerfilverifica(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSourceNamePerfil;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getAlertasSistema (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
            $dtcompetencia = $this->hcmParameters->getDtCompetencia();
            $cdoperador = $this->hcmParameters->getCdoperador();
            $exibeverifica = null;
            $agrupaverifica = null;
            $nrverificacao = null;
            $msgverificacao = null;
            foreach ($request->getFilterCriteria()->getConditions() as $condition) {
                if ($condition['columnName'] == 'IDEXIBEVERIFICA') {
                    $exibeverifica = $condition['value'];
                }
                if ($condition['columnName'] == 'IDAGRUPAVERIFICA') {
                    $agrupaverifica = $condition['value'];
                }
                if ($condition['columnName'] == 'NRVERIFICACAO') {
                    $nrverificacao = $condition['value'];
                }
                if ($condition['columnName'] == 'MSGVERIFICACAO') {
                    $msgverificacao = str_replace('%','',$condition['value']);
                }
            }
            $verificacoes = $this->verificacaoService->getVerificacoes(
                $nrorg, $nrorgPadrao, $dtcompetencia, $cdoperador, $exibeverifica, $agrupaverifica, $nrverificacao, $msgverificacao
            );
            $response->addDataSet(new DataSet('getAlertasSistema', $verificacoes));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}