<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class ESocialItensNaoPerRepository {

    protected  $connection;

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getItensParaEnvioS2190($params) {
        $sql = <<<SQL
SELECT PE.NRPESSOA as NRCHAVE,
       '' as NRTABELA,
       'I' as IDOPERACAO,
       PH.DTADMISSAOPRE as DTMESCOMPETENC
  FROM GPE_PESSOA PE
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE PE.NRORG = :P_NRORG
   AND PH.DTADMISSAOPRE IS NOT NULL
   AND NOT EXISTS (SELECT 1
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO = 'S-2190'
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = PE.NRORG
                      AND EC.NRCHAVE = PE.NRPESSOA
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                      AND EC.DTMESCOMPETENC <= PH.DTMESCOMPETENC)
   AND (PE.NRPESSOA = :P_NRPESSOA OR :P_NRPESSOA IS NULL)
 ORDER BY 1,4
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRPESSOA" => isset($params['NRPESSOA']) ? $params['NRPESSOA'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }

    public function getItensParaEnvioS2200($params) {
        try {
            $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       'I' as IDOPERACAO,
       GREATEST(NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC),NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC)) as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= NVL(:P_DTRESCISAOVINC,SYSDATE))
   AND VM.NRTIPOVINCULOM = 1 /*CLT*/
   AND NOT EXISTS (SELECT 1
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO = 'S-2200'
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = VM.NRORG
                      AND EC.NRCHAVE = VM.NRVINCULOM
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                      AND EC.DTMESCOMPETENC <= GREATEST(NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC),NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC))
                      AND NOT EXISTS(SELECT 1
                                       FROM ESO_ENVIOCADASTROS ECC
                                       JOIN ESO_EVENTO EV ON EV.NREVENTO = ECC.NREVENTO
                                                         AND EV.NMEVENTO = 'S-2299'
                                       /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                                      WHERE ECC.NRORG = VM.NRORG
                                        AND ECC.NRCHAVE = VM.NRVINCULOM
                                        AND ECC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                                        AND (ECC.IDSTATUS = 'C' OR ECC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                                        AND ECC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                                        AND ECC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                                        AND ECC.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND ECC.DTMESCOMPETENC <= GREATEST(NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC),NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC))))
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 1,5
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
                "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
                "P_DTRESCISAOVINC" => isset($params['DTRESCISAOVINC']) ? $params['DTRESCISAOVINC'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getItensParaEnvioS2205($params) {
        try {
            $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,    
       NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC) as DTMESCOMPETENC,
       'A' as IDOPERACAO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'MONTH'),-24)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'MONTH'),-12)
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND EXISTS (SELECT 1 /*Que existe evento de admissao*/
                 FROM ESO_ENVIOCADASTROS EC
                 JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                   AND EV.NMEVENTO IN ('S-2190','S-2200','S-2300')
                 /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                 JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                    AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                    AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                    AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                               WHERE H1.NRORG = EVH.NRORG
                                                                 AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                WHERE EC.NRORG = VM.NRORG
                  AND EC.NRCHAVE = VM.NRVINCULOM
                  AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                  AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                  AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                  AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                  AND EC.DTMESCOMPETENC < NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC)
                  AND EC.DTMESCOMPETENC = (SELECT MAX(ECC.DTMESCOMPETENC) 
                                             FROM ESO_ENVIOCADASTROS ECC 
                                             JOIN ESO_EVENTO EVV ON EVV.NREVENTO = ECC.NREVENTO
                                                                AND EVV.NMEVENTO IN ('S-2190','S-2200','S-2300')
                                            WHERE ECC.NRORG = EC.NRORG 
                                              AND ECC.NRESTRUTURAM = EC.NRESTRUTURAM
                                              AND ECC.NRCHAVE = EC.NRCHAVE
                                              AND ECC.NRENVIOEXCL IS NULL 
                                              AND ECC.IDATIVO = 'S' 
                                              AND ECC.NRTPAMBIENTE = EC.NRTPAMBIENTE ))
   AND NOT EXISTS (SELECT 1 /*NAO EXISTE EVENTO DE DESLIGAMENTO CONCLUIDO*/
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO = 'S-2299'
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = VM.NRORG
                      AND EC.NRCHAVE = VM.NRVINCULOM
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                      AND EC.DTMESCOMPETENC <= NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC))
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 1,3
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
                "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getItensParaEnvioS2206($params) {
        try {
            $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       GREATEST(NVL(SL.DTALTESALARIO, '01/01/1000'), NVL(AE.DTINIESCALA, '01/01/1000'), NVL(MOV.DTINIMOVIMENT, '01/01/1000'), NVL(VM.DTESOCIAL, '01/01/1000'), NVL(AO.DTINIOCUPACAO, '01/01/1000')) as DTMESCOMPETENC,
       SL.DTALTESALARIO,AE.DTINIESCALA,MOV.DTINIMOVIMENT,VM.DTESOCIAL,
       'A' as IDOPERACAO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'MONTH'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)                                                 
  LEFT JOIN GPE_ALTESALARIO SL ON SL.NRVINCULOM = VM.NRVINCULOM
                              AND SL.NRORG = VM.NRORG
                              AND SL.DTALTESALARIO = (SELECT MAX(SLL.DTALTESALARIO) FROM GPE_ALTESALARIO SLL
                                                     WHERE SLL.NRVINCULOM = SL.NRVINCULOM
                                                       AND SLL.NRORG = SL.NRORG
                                                       AND SLL.DTALTESALARIO <= LAST_DAY(:P_DTMESCOMPETENC))
                              AND SL.NROCORRENCIA =  (SELECT MAX(NROCORRENCIA) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO = SL.DTALTESALARIO
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
  LEFT JOIN GPE_ALTEESCALA AE ON AE.NRORG = VH.NRORG
                             AND AE.NRVINCULOM = VH.NRVINCULOM
                             AND (AE.DTFIMESCALA IS NULL OR AE.DTFIMESCALA > :P_DTMESCOMPETENC)
                             AND AE.DTINIESCALA = (SELECT MAX(DTINIESCALA) FROM GPE_ALTEESCALA
                                                    WHERE GPE_ALTEESCALA.DTINIESCALA <= LAST_DAY(:P_DTMESCOMPETENC)
                                                      AND (GPE_ALTEESCALA.DTFIMESCALA IS NULL OR GPE_ALTEESCALA.DTFIMESCALA > LAST_DAY(:P_DTMESCOMPETENC))
                                                      AND GPE_ALTEESCALA.NRVINCULOM = AE.NRVINCULOM
                                                      AND GPE_ALTEESCALA.NRORG = AE.NRORG)
  LEFT JOIN GPE_ALTEOCUPACAO AO ON AO.NRORG = VH.NRORG
                                AND AO.NRVINCULOM = VH.NRVINCULOM
                                AND (AO.DTFIMOCUPACAO IS NULL OR AO.DTFIMOCUPACAO > :P_DTMESCOMPETENC)
                                AND AO.DTINIOCUPACAO = (SELECT MAX(DTINIOCUPACAO) FROM GPE_ALTEOCUPACAO
                                                         WHERE GPE_ALTEOCUPACAO.DTINIOCUPACAO <= LAST_DAY(:P_DTMESCOMPETENC)
                                                           AND (GPE_ALTEOCUPACAO.DTFIMOCUPACAO IS NULL OR GPE_ALTEOCUPACAO.DTFIMOCUPACAO > LAST_DAY(:P_DTMESCOMPETENC))
                                                           AND GPE_ALTEOCUPACAO.NRVINCULOM = AE.NRVINCULOM
                                                           AND GPE_ALTEOCUPACAO.NRORG = AE.NRORG)
  LEFT JOIN (SELECT MO.NRORG, MO.NRESTRUTURAM, MO.NRVINCULOM, MO.DTFIMMOVIMENT, EM.NMESTRUTURAM, EH.CDCNPJESTRUT, MO.DTINIMOVIMENT
               FROM GPE_MOVIMENTACAO MO
               JOIN ESTRUTURAM EM ON EM.NRORG = MO.NRORG
                                 AND EM.NRESTRUTURAM = MO.NRESTRUTURAM
               LEFT JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                                 AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                                 AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                           WHERE H1.NRORG = EH.NRORG
                                                             AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                             AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE MO.NRORG = :P_NRORG
                AND MO.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
                AND MO.NRTIPOTRANSFER = 1
                AND MO.NRTPMOVTRANSFM = 8
                AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                AND MO.DTINIMOVIMENT <= LAST_DAY(:P_DTMESCOMPETENC)) MOV ON MOV.NRORG = VH.NRORG
                                                                                                   AND MOV.NRVINCULOM = VH.NRVINCULOM 
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND VM.NRTIPOVINCULOM = 1 /*CLT*/
   AND EXISTS (SELECT 1 /*Que existe evento de admissao*/
                 FROM ESO_ENVIOCADASTROS EC
                 JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                   AND EV.NMEVENTO IN ('S-2190','S-2200')
                 /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                 JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                    AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                    AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                    AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                               WHERE H1.NRORG = EVH.NRORG
                                                                 AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                WHERE EC.NRORG = VM.NRORG
                  AND EC.NRCHAVE = VM.NRVINCULOM
                  AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                  AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                  AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                  AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                  AND EC.DTMESCOMPETENC < NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC)
                  AND EC.DTMESCOMPETENC = (SELECT MAX(ECC.DTMESCOMPETENC) 
                                             FROM ESO_ENVIOCADASTROS ECC 
                                             JOIN ESO_EVENTO EVV ON EVV.NREVENTO = ECC.NREVENTO
                                                                AND EVV.NMEVENTO IN ('S-2190','S-2200')
                                            WHERE ECC.NRORG = EC.NRORG 
                                              AND ECC.NRESTRUTURAM = EC.NRESTRUTURAM
                                              AND ECC.NRCHAVE = EC.NRCHAVE
                                              AND ECC.NRENVIOEXCL IS NULL 
                                              AND ECC.IDATIVO = 'S' 
                                              AND ECC.NRTPAMBIENTE = EC.NRTPAMBIENTE ))
   AND NOT EXISTS (SELECT 1 /*NAO EXISTE EVENTO DE DESLIGAMENTO CONCLUIDO*/
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO = 'S-2299'
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = VM.NRORG
                      AND EC.NRCHAVE = VM.NRVINCULOM
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                      AND EC.DTMESCOMPETENC <= NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC))
   AND (SL.DTALTESALARIO IS NOT NULL OR AE.DTINIESCALA IS NOT NULL OR
        MOV.DTINIMOVIMENT IS NOT NULL OR AO.DTINIOCUPACAO IS NOT NULL OR
        VM.DTESOCIAL BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC))                      
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 1,3
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
                "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
                "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
            );
            
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getItensParaEnvioS2210($params) {
//         $sql = <<<SQL
// SELECT VI.NRVITIMA as NRCHAVE,
//       VH.NRVINCULOM,
//       'I' as IDOPERACAO,
//       OC.DTHROCORRENCIA as DTMESCOMPETENC
//   FROM SST_VITIMA VI
//   /*Comunicação de Acidente de Trabalho.*/
//   JOIN SST_OCORRENCIA OC ON OC.NRORG = VI.NRORG
//                         AND OC.NROCORRENCIA = VI.NROCORRENCIA
//   /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
//   LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
//               FROM ESO_ENVIOCADASTROS EC
//               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
//                                  AND EV.NMEVENTO = 'S-2210'
//               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
//               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
//                                   AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
//                                   AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
//                                   AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
//                                                              WHERE H1.NRORG = EVH.NRORG
//                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
//                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
//               WHERE EC.NRORG = :P_NRORG
//                 AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
//                 AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
//                 AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
//                 AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
//               GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VI.NRVITIMA
//   JOIN GPE_PESSOA PS ON PS.NRPARCNEGOCIO = VI.NRPARCNEGOCIO 
//                     AND PS.NRORG = VI.NRORG
//   JOIN GPE_VINCULOM VM ON VM.NRPESSOA = PS.NRPESSOA 
//                       AND VM.NRORG = PS.NRORG
//   JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
//                       AND VH.NRVINCULOM = VM.NRVINCULOM
//                       AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
//                                                 WHERE H1.NRVINCULOM = VH.NRVINCULOM
//                                                   AND H1.NRORG = VH.NRORG
//                                                   AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
//                                                   AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)      
//  WHERE VI.NRORG = :P_NRORG 
//   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
//   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
//   AND (VI.NRVITIMA = :P_NRVITIMA OR :P_NRVITIMA IS NULL)
//  ORDER BY 1,4
// SQL;
        $sql = <<<SQL
SELECT VI.NRVITIMA as NRCHAVE,
       VH.NRVINCULOM,
       'I' as IDOPERACAO,
       OC.DTHROCORRENCIA as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN SST_VITIMA VI ON VI.NRPARCNEGOCIO = PE.NRPARCNEGOCIO 
                    AND VI.NRORG = PE.NRORG
  /*Comunicação de Acidente de Trabalho.*/
  JOIN SST_OCORRENCIA OC ON OC.NRORG = VI.NRORG
                        AND OC.NROCORRENCIA = VI.NROCORRENCIA
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2210'
               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                  AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                  AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                  AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                             WHERE H1.NRORG = EVH.NRORG
                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VI.NRVITIMA
 WHERE VM.NRORG = :P_NRORG 
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 1,4
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            //"P_NRVITIMA" => isset($params['NRVITIMA']) ? $params['NRVITIMA'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2220($params) {
        $sql = <<<SQL
SELECT FM.NRFICHAMEDICA as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM SST_FICHAMEDICA FM
  JOIN GPE_VINCULOM VM ON FM.NRVINCULOM = VM.NRVINCULOM AND VM.NRORG = FM.NRORG
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2220'
               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                  AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                  AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                  AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                             WHERE H1.NRORG = EVH.NRORG
                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE FM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND (FM.NRFICHAMEDICA = :P_NRFICHAMEDICA OR :P_NRFICHAMEDICA IS NULL)
 ORDER BY 1,5
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRFICHAMEDICA" => isset($params['NRFICHAMEDICA']) ? $params['NRFICHAMEDICA'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2221($params) {
        $sql = <<<SQL
SELECT EFM.NREXAMEFICHAMED as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       NVL(EFM.DTREALIZACAOEXAME,FM.DTCONCLFICHAMED) as DTMESCOMPETENC
  FROM SST_EXAMEFICHAMED EFM
  JOIN SST_EXAME EXA ON EXA.NRORG = EFM.NRORG
                    AND EXA.NREXAME = EFM.NREXAME
  JOIN SST_FICHAMEDICA FM ON FM.NRORG = EFM.NRORG
                         AND FM.NRFICHAMEDICA = EFM.NRFICHAMEDICA
  JOIN GPE_VINCULOM VM ON VM.NRORG = FM.NRORG
                      AND VM.NRVINCULOM = FM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2221'
               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                  AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                  AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                  AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                             WHERE H1.NRORG = EVH.NRORG
                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = EFM.NREXAMEFICHAMED
 WHERE EFM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND NVL(EFM.DTREALIZACAOEXAME,FM.DTCONCLFICHAMED) BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
   AND EXA.NRTIPOEXAME = 7 /*Tipo Toxicológico*/
   AND (EFM.NREXAMEFICHAMED = :P_NREXAMEFICHAMED OR :P_NREXAMEFICHAMED IS NULL)
 ORDER BY 2,5
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NREXAMEFICHAMED" => isset($params['NREXAMEFICHAMED']) ? $params['NREXAMEFICHAMED'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2230($params) {
        $sql = <<<SQL
/*Afastamentos com data inicial na competência e sem data final ou data final posterior a competência*/
SELECT ASF.NRALTESITUFUNC as NRCHAVE,
       'I' as IDOPERACAO,
       NULL as NRTABELA,
       ASF.DTINISITUFUNC as DTMESCOMPETENC,
       ASF.NRVINCULOM
  FROM GPE_ALTESITUFUNC ASF
  JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Situação Funcional*/
  JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
  JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                        AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                              WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
 WHERE ASF.NRORG = :P_NRORG
   AND ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
   AND ASF.DTINISITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
   AND (ASF.DTFIMSITUFUNC IS NULL OR ASF.DTFIMSITUFUNC > LAST_DAY(:P_DTMESCOMPETENC))
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
        OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
   AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
   AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
   AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
UNION ALL
/*Afastamentos com data inicial e data final na competência*/
SELECT ASF.NRALTESITUFUNC as NRCHAVE,
       'I' as IDOPERACAO,
       NULL as NRTABELA,
       ASF.DTINISITUFUNC as DTMESCOMPETENC,
       ASF.NRVINCULOM
  FROM GPE_ALTESITUFUNC ASF
  JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Situação Funcional*/
  JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
  JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                        AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                              WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
 WHERE ASF.NRORG = :P_NRORG
   AND ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
   AND ASF.DTINISITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
   AND ASF.DTFIMSITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
        OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
   AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
   AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
   AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
UNION ALL
/*Afastamentos com data inicial anterior à competência e data final na competência*/
SELECT ASF.NRALTESITUFUNC as NRCHAVE,
       'I' as IDOPERACAO,
       NULL as NRTABELA,
       ASF.DTFIMSITUFUNC as DTMESCOMPETENC,
       ASF.NRVINCULOM
  FROM GPE_ALTESITUFUNC ASF
  JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Situação Funcional*/
  JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
  JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                        AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                              WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
 WHERE ASF.NRORG = :P_NRORG
   AND ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
   AND ASF.DTINISITUFUNC < :P_DTMESCOMPETENC
   AND ASF.DTFIMSITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
        OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
   AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
   AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
   AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
UNION ALL
/*Afastamentos com data final na competência para vínculos que iniciaram o eSocial afastados*/
SELECT ASF.NRALTESITUFUNC as NRCHAVE,
       'I' as IDOPERACAO,
       NULL as NRTABELA,
       ASF.DTFIMSITUFUNC as DTMESCOMPETENC,
       ASF.NRVINCULOM
  FROM GPE_ALTESITUFUNC ASF
  JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Situação Funcional*/
  JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
  JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                        AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                              WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
 WHERE ASF.NRORG = :P_NRORG
   AND ASF.DTINISITUFUNC < TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
   AND ASF.DTFIMSITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
   AND ASF.DTFIMSITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
        OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
   AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
   AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
   AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
UNION ALL
/*Gozo de férias como afastamento*/
/*Não considera IDENVIAESOCIAL pois gozo de férias ou recesso é sempre obrigatório o envio para o eSocial*/
SELECT GF.NRGOZOFERIAS as NRCHAVE,
       'I' as IDOPERACAO,
       2 as NRTABELA,
       GF.DTINIGOZOFERIAS as DTMESCOMPETENC,
       FE.NRVINCULOM
  FROM FPA_GOZOFERIAS GF
  JOIN FPA_FERIAS FE ON FE.NRORG = GF.NRORG
                    AND FE.NRFERIAS = GF.NRFERIAS
  JOIN GPE_VINCULOM VM ON FE.NRORG = VM.NRORG
                      AND FE.NRVINCULOM = VM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE GF.NRORG = :P_NRORG
   AND (GF.DTINIGOZOFERIAS BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC) OR
        GF.DTFIMGOZOFERIAS BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC))
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (GF.NRGOZOFERIAS = :P_NRGOZOFERIAS OR :P_NRGOZOFERIAS IS NULL)
   AND (FE.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND  EXISTS (SELECT 1 
                 FROM FPA_ITECALCFOLHA ITC
                 JOIN FPA_CALCULOFOLHA CFL ON CFL.NRORG = ITC.NRORG
                                          AND CFL.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                          AND CFL.NRTIPOMOVIMENT = 2
                                          AND CFL.NRTPMODALIDCAL = 1
            LEFT JOIN ESO_PARAMETRO ESOPARAMS ON ESOPARAMS.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                             AND ESOPARAMS.NRORG = VH.NRORG 
                WHERE ITC.NRVINCULOM = VH.NRVINCULOM
                  AND ITC.NRORG = VH.NRORG
                  AND ITC.NREVENTOM = '3030'
                  AND CFL.DTOCORRENCIA = (SELECT MAX(CF.DTOCORRENCIA)
                                            FROM FPA_ITECALCFOLHA IT
                                            JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IT.NRORG
                                                                    AND CF.NRCALCULOFOLHA = IT.NRCALCULOFOLHA
                                                                    AND CF.NRTIPOMOVIMENT = 2
                                                                    AND CF.NRTPMODALIDCAL = 1
                                           WHERE IT.NRORG      = ITC.NRORG
                                             AND IT.NRVINCULOM = ITC.NRVINCULOM
                                             AND IT.NREVENTOM = '3030')   
                  AND((ESOPARAMS.IDAPROVACALCFERIAS = 'S'AND EXISTS(SELECT 1
                                                                    FROM FPA_ITECALCFOLHA ITFEC 
                                                                   WHERE ITFEC.NRORG = ITC.NRORG
                                                                     AND ITFEC.NRVINCULOM = ITC.NRVINCULOM
                                                                     AND ITFEC.NREVENTOM = 3035
                                                                     AND ITFEC.NRCALCULOFOLHA = CFL.NRCALCULOFOLHA)) OR NVL(ESOPARAMS.IDAPROVACALCFERIAS, 'N') = 'N'))
 ORDER BY 4,5
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_INIVALIDPRODREST" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDPRODUCAO" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_NRTPAMBIENTE" => $params['NRTPAMBIENTE'],
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
            "P_NRALTESITUFUNC" => isset($params['NRALTESITUFUNC']) ? $params['NRALTESITUFUNC'] : null,
            "P_NRGOZOFERIAS" => isset($params['NRGOZOFERIAS']) ? $params['NRGOZOFERIAS'] : null,
        );
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2240($params) {
        $sql = <<<SQL
SELECT MO.NRMOVIMENTACAO as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       MO.DTINIMOVIMENT as DTMESCOMPETENC
  FROM GPE_MOVIMENTACAO MO    
  JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = MO.NRVINCULOM AND MO.NRORG = VM.NRORG
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN SST_RELESTRUTRISCO RTR ON RTR.NRORG = VH.NRORG AND RTR.NRESTRUTURAM = VH.NRESTRUTGEREN                                                 
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2240'
               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                  AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                  AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                  AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                             WHERE H1.NRORG = EVH.NRORG
                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = MO.NRMOVIMENTACAO
 WHERE MO.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM   
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND MO.DTINIMOVIMENT BETWEEN :P_DTMESCOMPETENC AND SYSDATE
   AND (MO.NRMOVIMENTACAO = :P_NRMOVIMENTACAO OR :P_NRMOVIMENTACAO IS NULL)
 ORDER BY 5,2
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRMOVIMENTACAO" => isset($params['NRMOVIMENTACAO']) ? $params['NRMOVIMENTACAO'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2241($params) {
       
        $sql = <<<SQL
SELECT MO.NRMOVIMENTACAO as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_MOVIMENTACAO MO 
  JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = MO.NRVINCULOM AND MO.NRORG = VM.NRORG
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN SST_RELESTRUTRISCO RTR ON RTR.NRORG = VH.NRORG AND RTR.NRESTRUTURAM = VH.NRESTRUTGEREN 
  JOIN SST_FATORRISCO FR ON FR.NRFATORRISCO = RTR.NRFATORRISCO AND FR.NRORG       = RTR.NRORG                                                
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2241'
               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                  AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                  AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                  AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                             WHERE H1.NRORG = EVH.NRORG
                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = MO.NRMOVIMENTACAO
 WHERE MO.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM   
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND (FR.IDGERAINSAL = 'S' OR FR.IDGERAPERIC = 'S' OR FR.IDGERAAPOSESPEC = 'S')        
   --AND (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)   
   AND (MO.NRMOVIMENTACAO = :P_NRMOVIMENTACAO OR :P_NRMOVIMENTACAO IS NULL)
 ORDER BY 5,2
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRMOVIMENTACAO" => isset($params['NRMOVIMENTACAO']) ? $params['NRMOVIMENTACAO'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2250($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       'I' as IDOPERACAO,
       NVL(VM.DTAVISOPREVIO,VM.DTCANCAVIPRE) as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN GPE_TIPOAVISOPRE TPA ON TPA.NRTPAVIPRE = VM.NRTPAVIPRE
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   /*QUE TENHA DATA DE RESCISAO E AVISO PRÉVIO E NAO SEJA INDENIZADO OU TENHA CANCELAMENTO DE AVISO PREVIO*/
   AND ((VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC AND
         VM.DTAVISOPREVIO BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC) AND
         (TPA.CDESOCIAL IN (1,2,4,5,6) AND VM.DTRESCISAOVINC <> VM.DTAVISOPREVIO) /*SEM JUSTA CAUSA EMPREGADOR E DATAS IGUAIS EH INDENIZADO - NAO DEVE IR NO S-2250*/)
        OR VM.DTCANCAVIPRE BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
       )
   AND EXISTS (SELECT 1 /*Que existe evento de admissao*/
                 FROM ESO_ENVIOCADASTROS EC
                 JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                   AND EV.NMEVENTO IN ('S-2200','S-2300')
                 /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                 JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                    AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                    AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                    AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                               WHERE H1.NRORG = EVH.NRORG
                                                                 AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                WHERE EC.NRORG = VM.NRORG
                  AND EC.NRCHAVE = VM.NRVINCULOM
                  AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                  AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                  AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                  AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE)
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 4,2
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2260($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,    
       'I' as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND VM.NRTIPOVINCULOM = 1 /*CLT*/
   AND VM.NRVINCULOEMPREG = 25 /*VINCULOM INTERMITENTE*/
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 4,1
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2298($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VM.DTRETORNO as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2298'
               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                  AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                  AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                  AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                             WHERE H1.NRORG = EVH.NRORG
                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND VM.DTRETORNO IS NOT NULL
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 5,1
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2299($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       'I' as IDOPERACAO,
       NVL(MO.DTFIMMOVIMENT,VM.DTRESCISAOVINC) as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
   /*Transferencia de saida*/
   LEFT JOIN GPE_MOVIMENTACAO MO ON MO.NRORG = VM.NRORG
                                AND MO.NRVINCULOM = VM.NRVINCULOM
                                AND MO.NRESTRUTURAM = :P_NRESTRUTURAM
                                AND MO.NRTPMOVTRANSFM = 9
                                AND MO.DTFIMMOVIMENT BETWEEN ADD_MONTHS(:P_DTMESCOMPETENC, '-1') AND LAST_DAY(:P_DTMESCOMPETENC)
   LEFT JOIN ESTRUTURAH EH ON EH.NRORG = MO.NRORG
                          AND EH.NRESTRUTURAM = MO.NRESTRUTURAM
                          AND EH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                     WHERE H1.NRORG = EH.NRORG
                                                       AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                       AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
   /*Transferencia de entrada*/
   LEFT JOIN GPE_MOVIMENTACAO ME ON ME.NRORG = MO.NRORG
                                AND ME.NRVINCULOM = MO.NRVINCULOM
                                AND ME.NRTIPOTRANSFER = MO.NRTIPOTRANSFER
                                AND ME.NRTIPOESTRUTURA = MO.NRTIPOESTRUTURA
                                AND ME.NRTPMOVTRANSFM = 8
                                AND ME.DTINIMOVIMENT > MO.DTFIMMOVIMENT
   LEFT JOIN ESTRUTURAH EE ON EE.NRORG = ME.NRORG
                          AND EE.NRESTRUTURAM = ME.NRESTRUTURAM
                          AND EE.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                     WHERE H1.NRORG = EE.NRORG
                                                       AND H1.NRESTRUTURAM = EE.NRESTRUTURAM
                                                       AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   /*Rescindidos ou transferidos para outro empregador*/
   AND ((VH.NRESTRUTLEGAL = :P_NRESTRUTURAM AND VM.DTRESCISAOVINC BETWEEN ADD_MONTHS(:P_DTMESCOMPETENC, '-1') AND LAST_DAY(:P_DTMESCOMPETENC)) OR
        (MO.NRMOVIMENTACAO IS NOT NULL AND SUBSTR(REPLACE(REPLACE(REPLACE(EE.CDCNPJESTRUT,'.'),'/'),'-'),1,8) <> SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8)))
   /*AND EXISTS (SELECT 1 /*Que existe evento de admissao*
                 FROM ESO_ENVIOCADASTROS EC
                 JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                   AND EV.NMEVENTO IN ('S-2190','S-2200')
                 /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*
                 JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                    AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                    AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                    AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                               WHERE H1.NRORG = EVH.NRORG
                                                                 AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                 AND H1.DTMESCOMPETENC <= P_DTMESCOMPETENC)
                WHERE EC.NRORG = VM.NRORG
                  AND EC.NRCHAVE = VM.NRVINCULOM
                  AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*
                  AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*
                  AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*
                  AND EC.NRTPAMBIENTE = P_NRTPAMBIENTE)*/
   AND (EXISTS (SELECT 1 
                 FROM FPA_ITECALCFOLHA ITC
                 JOIN FPA_CALCULOFOLHA CFL ON CFL.NRORG = ITC.NRORG
                                          AND CFL.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                          AND CFL.NRTIPOMOVIMENT = 1
                                          AND CFL.NRTPMODALIDCAL = 1
            LEFT JOIN ESO_PARAMETRO ESOPARAMS ON ESOPARAMS.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                             AND ESOPARAMS.NRORG = VH.NRORG 
                WHERE ITC.NRVINCULOM = VH.NRVINCULOM
                  AND ITC.NRORG = VH.NRORG
                  AND ITC.NREVENTOM = '3050'
                  AND CFL.DTOCORRENCIA = (SELECT MAX(CF.DTOCORRENCIA)
                                            FROM FPA_ITECALCFOLHA IT
                                            JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IT.NRORG
                                                                    AND CF.NRCALCULOFOLHA = IT.NRCALCULOFOLHA
                                                                    AND CF.NRTIPOMOVIMENT = 1
                                                                    AND CF.NRTPMODALIDCAL = 1
                                           WHERE IT.NRORG      = ITC.NRORG
                                             AND IT.NRVINCULOM = ITC.NRVINCULOM
                                             AND IT.NREVENTOM = '3050'
                                             AND IT.DTOCORRENCIA <= ADD_MONTHS(LAST_DAY(VM.DTRESCISAOVINC),1))   
                  AND((ESOPARAMS.IDAPROVACALCRESC = 'S'AND EXISTS(SELECT 1
                                                                    FROM FPA_ITECALCFOLHA ITFEC 
                                                                   WHERE ITFEC.NRORG = ITC.NRORG
                                                                     AND ITFEC.NRVINCULOM = ITC.NRVINCULOM
                                                                     AND ITFEC.NREVENTOM = 3035
                                                                     AND ITFEC.NRCALCULOFOLHA = CFL.NRCALCULOFOLHA)) OR NVL(ESOPARAMS.IDAPROVACALCRESC, 'N') = 'N')) OR MO.DTFIMMOVIMENT IS NOT NULL)
   AND VM.NRTIPOVINCULOM = 1 /*CLT*/
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 4,1
SQL;

        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => array_key_exists('DTMESCOMPETENCIA', $params) ? $params['DTMESCOMPETENCIA'] : $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        
        return $this->connection->fetchAll($sql, $params);
    }

    public function getItensParaEnvioS2300($params) {
        try {
            $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       'I' as IDOPERACAO,
       GREATEST(NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC),NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC)) as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= NVL(:P_DTRESCISAOVINC,SYSDATE))
   AND VM.NRTIPOVINCULOM IN (2,3,10) /*Estagiario, Socio, Autonomo*/
   AND NOT EXISTS (SELECT 1
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO = 'S-2300'
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = VM.NRORG
                      AND EC.NRCHAVE = VM.NRVINCULOM
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                      AND EC.DTMESCOMPETENC <= GREATEST(NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC),NVL(PE.DTESOCIAL,PH.DTMESCOMPETENC)))
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 1
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
                "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
                "P_DTRESCISAOVINC" => isset($params['DTRESCISAOVINC']) ? $params['DTRESCISAOVINC'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getItensParaEnvioS2306($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       'A' as IDOPERACAO,
       NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC) as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND  VM.NRTIPOVINCULOM IN (2,3,10) --Estagiario, Socio, Autonomo
   AND EXISTS (SELECT 1 /*Que existe evento de admissao*/
                 FROM ESO_ENVIOCADASTROS EC
                 JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                   AND EV.NMEVENTO IN ('S-2190','S-2300')
                 /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                 JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                    AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                    AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                    AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                               WHERE H1.NRORG = EVH.NRORG
                                                                 AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                WHERE EC.NRORG = VM.NRORG
                  AND EC.NRCHAVE = VM.NRVINCULOM
                  AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                  AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                  AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                  AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                  AND EC.DTMESCOMPETENC < NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC))
   AND NOT EXISTS (SELECT 1 /*NAO EXISTE EVENTO DE DESLIGAMENTO CONCLUIDO*/
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO = 'S-2399'
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = VM.NRORG
                      AND EC.NRCHAVE = VM.NRVINCULOM
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                      AND EC.DTMESCOMPETENC <= NVL(VM.DTESOCIAL,VH.DTMESCOMPETENC))
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 4,1
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getItensParaEnvioS2399($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       VH.NRVINCULOM,
       'I' as IDOPERACAO,
       NVL(MO.DTFIMMOVIMENT,VM.DTRESCISAOVINC) as DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Transferencia de saida*/
  LEFT JOIN GPE_MOVIMENTACAO MO ON MO.NRORG = VM.NRORG
                               AND MO.NRVINCULOM = VM.NRVINCULOM
                               AND MO.NRESTRUTURAM = :P_NRESTRUTURAM
                               AND MO.NRTPMOVTRANSFM = 9
                               AND MO.DTFIMMOVIMENT BETWEEN ADD_MONTHS(:P_DTMESCOMPETENC, '-1') AND LAST_DAY(:P_DTMESCOMPETENC)
  LEFT JOIN ESTRUTURAH EH ON EH.NRORG = MO.NRORG
                         AND EH.NRESTRUTURAM = MO.NRESTRUTURAM
                         AND EH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                    WHERE H1.NRORG = EH.NRORG
                                                      AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                      AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Transferencia de entrada*/
  LEFT JOIN GPE_MOVIMENTACAO ME ON ME.NRORG = MO.NRORG
                               AND ME.NRVINCULOM = MO.NRVINCULOM
                               AND ME.NRTIPOTRANSFER = MO.NRTIPOTRANSFER
                               AND ME.NRTIPOESTRUTURA = MO.NRTIPOESTRUTURA
                               AND ME.NRTPMOVTRANSFM = 8
                               AND ME.DTINIMOVIMENT > MO.DTFIMMOVIMENT
  LEFT JOIN ESTRUTURAH EE ON EE.NRORG = ME.NRORG
                         AND EE.NRESTRUTURAM = ME.NRESTRUTURAM
                         AND EE.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                    WHERE H1.NRORG = EE.NRORG
                                                      AND H1.NRESTRUTURAM = EE.NRESTRUTURAM
                                                      AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   /*Rescindidos ou transferidos para outro empregador*/
   AND ((VH.NRESTRUTLEGAL = :P_NRESTRUTURAM AND VM.DTRESCISAOVINC BETWEEN ADD_MONTHS(:P_DTMESCOMPETENC, '-1') AND LAST_DAY(:P_DTMESCOMPETENC)) OR
        (MO.NRMOVIMENTACAO IS NOT NULL AND SUBSTR(REPLACE(REPLACE(REPLACE(EE.CDCNPJESTRUT,'.'),'/'),'-'),1,8) <> SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8)))
   /*AND EXISTS (SELECT 1 /*Que existe evento de admissao*
                 FROM ESO_ENVIOCADASTROS EC
                 JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                   AND EV.NMEVENTO = 'S-2300'
                 /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*
                 JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                    AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                    AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                    AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                               WHERE H1.NRORG = EVH.NRORG
                                                                 AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                 AND H1.DTMESCOMPETENC <= P_DTMESCOMPETENC)
                WHERE EC.NRORG = VM.NRORG
                  AND EC.NRCHAVE = VM.NRVINCULOM
                  AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*
                  AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*
                  AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*
                  AND EC.NRTPAMBIENTE = P_NRTPAMBIENTE)*/
   AND (EXISTS (SELECT 1 
                 FROM FPA_ITECALCFOLHA ITC
                 JOIN FPA_CALCULOFOLHA CFL ON CFL.NRORG = ITC.NRORG
                                          AND CFL.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                          AND CFL.NRTIPOMOVIMENT = 1
                                          AND CFL.NRTPMODALIDCAL = 1
            LEFT JOIN ESO_PARAMETRO ESOPARAMS ON ESOPARAMS.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                             AND ESOPARAMS.NRORG = VH.NRORG 
                WHERE ITC.NRVINCULOM = VH.NRVINCULOM
                  AND ITC.NRORG = VH.NRORG
                  AND ITC.NREVENTOM = '3050'
                  AND CFL.DTOCORRENCIA = (SELECT MAX(CF.DTOCORRENCIA)
                                            FROM FPA_ITECALCFOLHA IT
                                            JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IT.NRORG
                                                                    AND CF.NRCALCULOFOLHA = IT.NRCALCULOFOLHA
                                                                    AND CF.NRTIPOMOVIMENT = 1
                                                                    AND CF.NRTPMODALIDCAL = 1
                                           WHERE IT.NRORG      = ITC.NRORG
                                             AND IT.NRVINCULOM = ITC.NRVINCULOM
                                             AND IT.NREVENTOM = '3050'
                                             AND IT.DTOCORRENCIA <= ADD_MONTHS(LAST_DAY(VM.DTRESCISAOVINC),1))   
                  AND((ESOPARAMS.IDAPROVACALCRESC = 'S'AND EXISTS(SELECT 1
                                                                    FROM FPA_ITECALCFOLHA ITFEC 
                                                                   WHERE ITFEC.NRORG = ITC.NRORG
                                                                     AND ITFEC.NRVINCULOM = ITC.NRVINCULOM
                                                                     AND ITFEC.NREVENTOM = 3035
                                                                     AND ITFEC.NRCALCULOFOLHA = CFL.NRCALCULOFOLHA)) OR NVL(ESOPARAMS.IDAPROVACALCRESC, 'N') = 'N')) OR MO.DTFIMMOVIMENT IS NOT NULL)
   AND VM.NRTIPOVINCULOM IN (2,3,10) --Estagiario, Socio, Autonomo
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 4,1
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }

}