<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Geral\Repository\VinculoRepository;
use HCM\Geral\Services\VinculoServices;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Folha\Entities\VabLinhavinc;
use Folha\Entities\FpaVincevento;
use HCM\Util\Messages;
use Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh;
use Teknisa\Model\Gestaopessoas\Entities\GpeTipotransfer;
use Teknisa\Model\Gestaopessoas\Entities\GpeTpmovtransfm;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;
use HCM\FPA\Services\DependenteVincService;
use HCM\Geral\Services\MovimentacaoService;
use HCM\Geral\Services\AlteSituacaoFuncionalService;
use HCM\Geral\Services\AlteSalarioService;
use HCM\Geral\Services\AlteEscalaService;
use HCM\Geral\Services\AlteOcupacaoService;
use HCM\Geral\Services\ParcnegocioService;
use HCM\Geral\Services\PessoaService;
use HCM\Geral\Services\UploadFileService;
use HCM\Geral\Services\OperadorService;
use HCM\Geral\Services\SolicitacaoService;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\VAB\Services\ValeTransporteService;
use HCM\Util\Util;

class AdmissaoService {
    
    const TP_SALFAMILIA = 1;
    const TP_IRRF = 2;
    const TP_PENSAO = 3;

    protected $gpe_Vinculoh;
    /**
     * @var vinculoRepository
     */
    protected $vinculoRepository;

    public function __construct(EntityManager $entityManager, VinculoRepository $vinculoRepository, HCMParameters $hcmParameters,
    VinculoServices $vinculoServices,MovimentacaoService $movimentacaoService,AlteSituacaoFuncionalService $altesitufunc,
    AlteSalarioService $alteSalario, AlteEscalaService $alteEscala, AlteOcupacaoService $alteOcupacao,
    ParcnegocioService $parcNegocio,PessoaService $pessoaService,DependenteVincService $dependenteVincService,
    UploadFileService $uploadFoto,OperadorService $operadorService,SolicitacaoService $solicitacaoService, ValeTransporteService $valeTransporteService) {
        
        $this->entityManager = $entityManager;
        $this->vinculoRepository = $vinculoRepository;
        $this->hcmParameters = $hcmParameters;
        $this->vinculoServices = $vinculoServices;
        $this->movimentacaoService = $movimentacaoService;
        $this->altesitufunc = $altesitufunc;
        $this->alteSalario = $alteSalario;
        $this->alteEscala = $alteEscala;
        $this->alteOcupacao = $alteOcupacao;
        $this->parcNegocioService = $parcNegocio;
        $this->pessoaService = $pessoaService;
        $this->dependenteVincService = $dependenteVincService;
        $this->uploadFileService = $uploadFoto;
        $this->operadorService = $operadorService;
        $this->solicitacaoService = $solicitacaoService;
        $this->valeTransporteService = $valeTransporteService;
        
        $this->gpe_vinculomEntity = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->gpe_Vinculoh = Repositories::GPE_VINCULOH;
        $this->gpe_Vinculom = Repositories::GPE_VINCULOM;
        $this->gpe_LinhaVinc = Repositories::VAB_LINHAVINC;
        $this->beneficiario = Repositories::VAB_BENEFICIARIO;
        $this->fpa_vincEvento = Repositories::FPA_VINCEVENTO;

    }
    
    public function buscarPessoaCPF($nrcpfpessoa){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        return $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf($nrorg, $dtmescompetenc, $nrcpfpessoa);
    }
    
    public function procuraPorDependentes($nrparcnegocio) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorg_padrao = $this->hcmParameters->getNrorgPadrao();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        return $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaRelacionamentosPorParcnegocio($nrorg, $nrorg_padrao, $nrparcnegocio, $dtmescompetenc);
    }
    
    /**
     * @return int
     */
    
    public function admissaoColetiva($nrorg, $nrestrutlegal, $nrestrutgeren_list, $nrvinculom_list, $dtrescisao_ini, $dtrescisao_fin, $dtadmissao)
    {
        // Array de vínculos para readmissão
        $vinculos = array();
        $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $dtinicial = DateUtil::getDataDeString($dtrescisao_ini, DateUtil::FORMATO_BRASILEIRO, true);
        $dtfinal = DateUtil::getDataDeString($dtrescisao_fin, DateUtil::FORMATO_BRASILEIRO, true);
        $dtadmissao = DateUtil::getDataDeString($dtadmissao, DateUtil::FORMATO_BRASILEIRO, true);
        
        if ($nrvinculom_list == "A") {
            if ($nrestrutgeren_list == "A") {
                // Busca por estrutura legal
                $vinculos = $this->vinculoRepository->retornaRescindidosGeral($dtmescompetenc, $nrestrutlegal, null, $dtinicial, $dtfinal, 'ESTRUTURA',$nrestrutlegal, true);
                
                
            } else {
                // Busca por estrutura legal e gerencial
                foreach ($nrestrutgeren_list as $nrestrutgeren) {
                    $vinculos = array_merge(
                        $vinculos,
                        $this->vinculoRepository->retornaRescindidosGeral($dtmescompetenc, $nrestrutlegal, $nrestrutgeren, $dtinicial, $dtfinal, 'ESTRUTURA',null,true)
                    );
                }
            }
        } else {
            $nrvinculom_list = implode(',',array_map(function($vinculo){return $vinculo;},$nrvinculom_list));
            // Busca por lista de vínculos
            
            $vinculos = $this->vinculoRepository->retornaRescindidosGeral($dtmescompetenc, $nrvinculom_list, $nrvinculom_list, $dtinicial, $dtfinal, 'LISTA_VINCULO', $nrestrutlegal, true);
        }

        if (count($vinculos) > 0) {
            // Readmissão dos vínculos
            try {
                $this->entityManager->beginTransaction();
                /** @var $vinculo \GpevinculomEntity */
                foreach ($vinculos as $vinculo) {
                    $vinculoh = $vinculo->historico;
                    // Busca último salário
                    /** @var GpeAltesalario $altesalario */
                    $altesalario = $this->entityManager->getRepository(Repositories::GPE_ALTESALARIO)->findOneBy(array('nrorg' => $vinculo->NRORG,
                    'nrvinculom' => $vinculo->NRVINCULOM),(array('dtaltesalario' => 'DESC')));
                    // Busca última escala
                    /** @var GpeAlteescala $alteescala */
                    $alteescala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->findOneBy(array('nrorg' => $vinculo->NRORG,
                    'nrvinculom' => $vinculo->NRVINCULOM),(array('dtiniescala' => 'DESC')));
                    // Busca última ocupação
                    /** @var GpeAlteocupacao $alteocupacao */
                    $alteocupacao = $this->entityManager->getRepository(Repositories::GPE_ALTEOCUPACAO)->findOneBy(array('nrorg' => $vinculo->NRORG,
                    'nrvinculom' => $vinculo->NRVINCULOM),(array('dtiniocupacao' => 'DESC')));
                    // Busca vales transportes
                    $linhas = VabLinhavinc::getObjectsByWhere(
                        'NRORG = :NRORG AND NRVINCULOM = :NRVINCULOM AND (DTFIMLINHAVINC IS NULL OR DTFIMLINHAVINC >= :DTADMISSAO)',
                        array('NRORG' => $vinculo->NRORG, 'NRVINCULOM' => $vinculo->NRVINCULOM, 'DTADMISSAO' => $dtadmissao)
                    );
                    // Busca eventos
                    $eventos = FpaVincevento::getObjectsByWhere(
                        'NRORG = :NRORG AND NRVINCULOM = :NRVINCULOM AND (DTFIMVIGENCIA IS NULL OR DTFIMVIGENCIA >= :DTADMISSAO)',
                        array('NRORG' => $vinculo->NRORG, 'NRVINCULOM' => $vinculo->NRVINCULOM, 'DTADMISSAO' => $dtadmissao)
                    );
                    // Busca dependentes
                    $dependentes = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaDependentesVinculo($vinculo->NRORG, $vinculo->NRVINCULOM, $dtmescompetenc->format('d/m/Y'));
                   
                    $dtprimadmissao = null;
                    // busca pessoa
                    $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array('nrorg' => $vinculo->NRORG, 'nrpessoa' => $vinculo->NRPESSOA));
                    $nrparcnegocio =  $pessoa->getNrparcnegocio();
                    
                    if($vinculo->DTPRIMADMISSAO){
                        $dtprimadmissao = DateUtil::getDataDeString($vinculo->DTPRIMADMISSAO, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
                    }
                    
                    // Verificar se pessoa possui outros vínculos ativos
                    $idmultvinc = GpeVinculoh::IDMULTVINC_UNICO;
                    $outrosVinculos = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaOutrosVinculosAtivosPorPessoa($vinculo->NRORG, $vinculo->NRPESSOA);
                    if($outrosVinculos){
                        $idmultvinc = GpeVinculoh::IDMULTVINC_PRIMARIO;
                    }
                    // foreach ($outrosVinculos as $vinculoPessoa) {
                    //     if (!$vinculoPessoa->getDtrescisaovinc()) {
                    //         $idmultvinc = GpeVinculoh::IDMULTVINC_PRIMARIO;
                    //     }
                    // }

                    // Insere vínculo
                    //$idpagpropfer13 = $vinculoh->getIdpagpropfer13() ? $vinculoh->getIdpagpropfer13() : null;
                    $matriculaEsocial = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->retonaUltimaMatriculaEsoIncrementada($nrorg, '151000063', $dtmescompetenc->format('d/m/Y'))['NOVO'];
                    $vinculom = $this->vinculoServices->novoMestre(
                        $vinculo->NRPESSOA,$nrorg, $dtadmissao, null, null, $vinculo->NRTIPOVINCULOM,
                        $vinculo->NRVINCULOEMPREG, $vinculo->NRTPADMISSAO,
                        $vinculo->NMVINCULOM,$dtprimadmissao,$dtadmissao,$dtadmissao
                        );

                    $this->vinculoServices->novoHistorico(
                        $nrorg,$vinculom->getNrvinculom(), $dtmescompetenc,GpeSitufuncm::SITUACAO_FUNCIONAL_NORMAL, $vinculoh->getNrcargo(),
                        $vinculoh->getNrescalatrabm(), $vinculoh->getIdcontribuisind(), 0, 0, $dtadmissao, $vinculoh->getIdremuneracao(),
                        $vinculoh->getIdtppagamento(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrtpmodalidsal(), $idmultvinc,
                        $vinculoh->getNrfuncao(), $vinculoh->getCdcontribindividual(), GpeTpmovtransfm::TPMOVTRANSF_ADMIS_EMPR_ANT, null,
                        $vinculoh->getNrestrutsind(), 'N', $matriculaEsocial, null
                    );
                    
                    // Insere movimentações
                    $movimentacoes = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaUltimasMovimentacoesVinculo($vinculo->NRORG, $vinculo->NRVINCULOM, $dtmescompetenc->format('d/m/Y'));
    
                    foreach($movimentacoes as $movimentacao){
                        $this->movimentacaoService->incluiMovimentacao($nrorg,$movimentacao['NRTIPOTRANSFER'],$vinculom->getNrvinculom(), $movimentacao['NRESTRUTURAM'], $dtadmissao, $movimentacao['NRTIPOESTRUTURA'], '2');
                    }
                    // if($vinculoh->getNrestrutsind()){
                    //     $this->movimentacaoService->incluiMovimentacao($nrorg,GpeTipotransfer::TIPOTRANSFLEGAL,$vinculom->getNrvinculom(), $vinculoh->getNrestrutsind(),$dtadmissao, 10, GpeTpmovtransfm::TPMOVTRANSF_ADMIS_EMPR_ANT);
                    //     //throw new \Exception(Messages::evaluateMessage(Messages::HCM00000_GER_VINC_FALTAESTRUTSIND,$vinculo->NMVINCULOM),10);
                    // }
                    // if($vinculoh->getNrestrutlegal()){
                    //     $this->movimentacaoService->incluiMovimentacao($nrorg,GpeTipotransfer::TIPOTRANSFLEGAL,$vinculom->getNrvinculom(), $vinculoh->getNrestrutlegal(),$dtadmissao, $this->hcmParameters->getTipoEstruturaLegalOrg(), GpeTpmovtransfm::TPMOVTRANSF_ADMIS_EMPR_ANT);                      
                    //     //throw new \Exception(Messages::evaluateMessage(Messages::HCM00000_GER_VINC_FALTAESTRUTSIND,$vinculo->NRVINCULOM),10);
                    // }
                    // if($vinculoh->getNrestrutgeren()){
                    //     $this->movimentacaoService->incluiMovimentacao($nrorg,GpeTipotransfer::TIPOTRANSFGEREN,$vinculom->getNrvinculom(), $vinculoh->getNrestrutgeren(),$dtadmissao, $this->hcmParameters->getTipoEstruturaGerencialOrg(), GpeTpmovtransfm::TPMOVTRANSF_ADMIS_EMPR_ANT);
                    //     //throw new \Exception(Messages::evaluateMessage(Messages::HCM00000_GER_VINC_FALTAESTRUTSIND,$vinculo->NRVINCULOM),10);
                    // }
                    
                    // Insere situação
                    $this->altesitufunc->nova($nrorg,$vinculom->getNrvinculom(), GpeSitufuncm::SITUACAO_FUNCIONAL_NORMAL, $dtadmissao);
                    // Insere salário
                    if (is_object($altesalario)) {
                        $this->alteSalario->nova($nrorg,$vinculom->getNrvinculom(),$dtadmissao, 1, 1, $altesalario->getVrsalario(), $altesalario->getIdtpsalario(), $altesalario->getNrtpmodalidsal());
                    }
                    // Insere escala
                    if (is_object($alteescala)) {
                        $this->alteEscala->nova($nrorg,$vinculom->getNrvinculom(),$alteescala->getNrescalatrabm(), $dtadmissao, $alteescala->getDsobservacao(), null, $alteescala->getNrhordiam(),
                                                $alteescala->getNrmotivoescala(), $alteescala->getNrturno(), $alteescala->setQthrlimitesephextra());
                        
                    }
                    // Insere ocupaçao
                    if (is_object($alteocupacao)) {
                        $this->alteOcupacao->nova($nrorg,$vinculom->getNrvinculom(),$alteocupacao->getNrocupacaom(), $dtadmissao, $alteocupacao->getNrtipoocupacao(), $alteocupacao->getNrtpmodalidsal());
                    }
                    // Insere Vales Transportes
                    /** @var VabLinhavinc $linhavinc */
                    foreach ($linhas as $linhavinc) {
                        $vale = new $this->gpe_LinhaVinc;
                        $vale->setNrorg($nrorg);
                        $vale->setNrvinculom($vinculom->getNrvinculom());
                        $vale->setNrlinhatarifa($linhavinc->getNrlinhatarifa());
                        $vale->setNrtipolinhavinc($linhavinc->getNrtipolinhavinc());
                        $vale->setNrintegtranspm($linhavinc->getNrintegtranspm());
                        $vale->setNrsentidolinha($linhavinc->getNrsentidolinha());
                        $vale->setQtlinhatarifa($linhavinc->getQtlinhatarifa());
                        $vale->setDtinilinhavinc($dtadmissao);
                        $vale->setDtfimlinhavinc($linhavinc->getDtfimlinhavinc() ? $linhavinc->getDtfimlinhavinc() : null);
                        $vale->setDtinclusao(DateUtil::getDataAtual());
                        $vale->setCdoperinclusao($this->hcmParameters->getCdoperador());
                        $vale->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                        $vale->setDtultatu(DateUtil::getDataAtual());
                        $vale->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $vale->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $vale->setNrlinhavinc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_LINHAVINC', 1, 12, true));
                        $this->entityManager->persist($vale);
                        unset($vale);
                    }
                    // Insere Evento
                    /** @var FpaVincevento $evento */
                    foreach ($eventos as $evento) {
                        $vincevento = new $this->fpa_vincEvento;
                        $vincevento->setNrorg($nrorg);
                        $vincevento->setNrvinculom($vinculom->getNrvinculom());
                        $vincevento->setNreventom($evento->getNreventom());
                        $vincevento->setDtinivigencia($dtadmissao);
                        $vincevento->setDtfimvigencia($evento->getDtfimvigencia() ? $evento->getDtfimvigencia() : null);
                        $vincevento->setVrpercent($evento->getVrpercent());
                        $vincevento->setVrmultiplic($evento->getVrmultiplic());
                        $vincevento->setVrdivide($evento->getVrdivide());
                        $vincevento->setVreventofixo($evento->getVreventofixo());
                        $vincevento->setIdrefgerar($evento->getIdrefgerar());
                        $vincevento->setIdretreferencia($evento->getIdretreferencia());
                        $vincevento->setVrreffixo($evento->getVrreffixo());
                        $vincevento->setIdtipomovimento($evento->getIdtipomovimento());
                        $vincevento->setVrpisoevento($evento->getVrpisoevento());
                        $vincevento->setVrtetoevento($evento->getVrtetoevento());
                        $vincevento->setPeriodicidademes($evento->getPeriodicidademes());
                        $vincevento->setDsobservacao($evento->getDsobservacao());
                        $vincevento->setNrtpagrupacalc($evento->getNrtpagrupacalc());
                        $vincevento->setNrestruturam($evento->getNrestruturam());
                        $vincevento->setIdgeraafastado($evento->getIdgeraafastado());
                        $vincevento->setNrtipoincide($evento->getNrtipoincide());
                        $vincevento->setNrvalorreferenciam($evento->getNrvalorreferenciam() ? $evento->getNrvalorreferenciam() : null);
                        $vincevento->setDtinclusao(DateUtil::getDataAtual());
                        $vincevento->setCdoperinclusao($this->hcmParameters->getCdoperador());
                        $vincevento->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                        $vincevento->setDtultatu(DateUtil::getDataAtual());
                        $vincevento->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $vincevento->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $vincevento->setNrvincevento(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_VINCEVENTO', 1, 12, true));
                        $this->entityManager->persist($vincevento);
                        unset($vincevento);
                        
                    }
                    
                    // insere dependentes
                    if($dependentes){
                        foreach($dependentes as $dependente){
                            $dependente = $this->verificaIndex($dependente);
                            $dependente['NMPESSOA'] = Util::removeCaracteresEspeciais($dependente['NMPESSOA']);
                            $dependente['NMPESSOA'] = strtoupper($dependente['NMPESSOA']);
                            $dependente['NRCPFPESSOA'] = str_replace(array('.','/','-'),'',$dependente['NRCPFPESSOA']);
                            
                            //INSERE DEPVINCULO SE FOR INFORMADO O TIPO DE DEPENDENTE
                            if(!empty($dependente['NRTIPODEPENDE'])){
                                if(is_array($dependente['NRTIPODEPENDE'])){
                                    foreach ($dependente['NRTIPODEPENDE'] as $depVinc) {
                                        $this->dependenteVincService->novo($nrorg, $dependente['NRPESSOA'], $depVinc, $vinculom->getNrvinculom());
                                    }
                                }else{
                                    $this->dependenteVincService->novo($nrorg, $dependente['NRPESSOA'], $dependente['NRTIPODEPENDE'], $vinculom->getNrvinculom());
                                }
                            }
                        }
                    }
                    
                    unset($alteocupacao);
                    unset($alteescala);
                    unset($altesalario);
                    unset($vinculoh);
                    unset($dependentes);
                }
                
                $this->entityManager->flush();
                $this->entityManager->commit();
                
            } catch (\Exception $e) {
                $this->entityManager->rollback();
                throw $e;
            }
        }else {
            throw new \Exception("Não foram encontrados vínculos rescindidos para o filtro informado",10);
        }
    }
    
    public function verificaMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, $dtmescompetenc, $nrestruturam, $cdmatriculaesocialante){
        return $this->vinculoServices->verificaMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, $dtmescompetenc, $nrestruturam, $cdmatriculaesocialante);
    }
    
    public function verificaIndex($indexs){
        
        $arrayindex = array('NMPESSOA','DSAPELIDOPESS','NRCERTNASCPES','DTNASCPESSOA','IDSEXOPESSOA','NRCONDFISPES','NRCPFPESSOA','NRCTPSPESSOA',
                'NRSERIECTPSPES','SGUFCTPSPES','DTCTPSPESSOA','NRPISPASEPPES','DTINSCPISPASEP','NRINSCINSSPES','NRINSCISSPES','CDESTACIVIL','NRCERTCASAPES',
                'DTCASAPESSOA','IDISENTITUELEI','NRTITUELEIPES','NRSECAELEIPES','NRZONAELEIPES','NRCARTHABPES','DTHABCARTPES','DTVALHABCARTPES','DSCATEGHABCART',
                'NRCATMILITARPE','NRCERTRESEPES','DSCATEGCERTRESE','DTCERTRESEPES','CDEXCERTRESEPES','NRRACAPESSOA','IDGRUPSANGUPES','IDFATORRHPES','CDPAIS',
                'SGESTADO','CDMUNICIPIO','NRRGPESSOA','CDEXRGPESSOA','SGUFRGPESSOA','DSLOCALEXRGPES','DTEXRGPESSOA','NRNACIONALID','NRGRAUINSTR','DSPRONOMETRAT',
                'NRANOCHEGAPAIS','NRDOCUESTRANG','IDPESNATURALIZA','DTNATURALIZAPES','IDPOSVISTOPERM','DTPERMAVISTO','NRCONSELHOREG','NRINSCRCONSREG',
                'DTEXCONSREG','SGUFCONSREG','NRLIVROCERTNASC','NRFOLHACERTNASC','DSCOMPLCERTNASC','NRREGCTPSPES','CDLOGRADOURO','NMBAIRROENDERECO',
                'CDTIPOENDERECO','NRCEPENDERECO','DSCOMPLEENDERECO','NRIMOVELENDERECO','DSENDERECO','DSREFERENCIAENDE','CDBANCO',
                'CDBANCO','CDAGENCIA','CDCONTCORR','NRTPCONTCORR','EMAIL','NRPESSOA','DTADMISSAOVINC','CDMATRICULA','NRTIPOVINCULOM',
                'NRVINCULOEMPREG','DTOPCAOFGTS','DTAPOSENTAFGTS','DTPRIMADMISSAO','NMVINCULOM','NRDIASCONTRATOEXP','NRDIASCONTRATOEXPPRO','DTEXAMEMEDICO',
                'IDCONTRIBUISIND','DTBASEFERIAS','IDREMUNERACAO','IDTPPAGAMENTO','IDMULTVINC','CDCONTRIBINDIVIDUAL','NRTPMOVTRANSFM','DTFIMCONTRDETERMIN',
                'NRTIPOTRANSFER_SINDICAL','NRESTRUTSIND','DTINIMOVIMENT','NRTIPOTRANSFER_LEGAL','NRESTRUTLEGAL','NRTIPOTRANSFER_GEREN','NRESTRUTGEREN', 'NRTIPOTRANSFER_ADICIONAL', 'NRESTRUTADICIONAL',
                'NRTPMOVTRANSF','NRSITUFUNCM','DTINISITUFUNC','QTHRLIMITESEPHEXTRA','QTHRLIMITESEPHEXTRA','DTALTESALARIO','NROCORRENCIA','NRTIPOALTERA','VRSALARIO',
                'CDTIPOSALARIO','NRTPMODALIDSAL','VRDIFERENCAREAL','VRPERCREAL','VRPERCALTERA','NRESCALATRABM','DTINIESCALA','DTFIMESCALA','NRTURNO','QTHRLIMITESEPHEXTRA',
                'NROCUPACAOM','DTINIOCUPACAO','NRTIPOOCUPACAO','NRTPMODALIDSAL','DTFIMOCUPACAO','INSERE_OPERADOR','NRTIPORELACIONA','DTFIMRELACIONAMENTO','NRTIPODEPENDE','NRNACIONALIDADE',
                'NRCLASSESTRANG', 'DSEXPDOCESTR', 'DTCHEGADAPAIS'
        );

        foreach($arrayindex as $index){
            if(!isset($indexs[$index]) || empty($indexs[$index])){
                $indexs[$index] = null;
            }
        }

        return $indexs;

    }
    
    public function insereProgOcorrenciaFrequencia($nrorg, $nrtipovinculom, $nrvinculom, $dtadmissao){
        try{
            $arrayProgOcorrencia = $this->entityManager->getRepository(REPOSITORIES::PTO_OCORRENCIADMISSAO)->findBy(array('nrorg' => $nrorg, 'nrtipovinculom' => $nrtipovinculom));
            foreach($arrayProgOcorrencia as $progOcorrencia){
                $itemProgOcorrencia = REPOSITORIES::PTO_PROGOCORRENCIA;
                $itemProgOcorrencia = new $itemProgOcorrencia;
                $itemProgOcorrencia->setNrorg($nrorg);
                $itemProgOcorrencia->setNrprogocorrencia(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_PROGOCORRENCIA', 1, 12, true));
                $itemProgOcorrencia->setNrvinculom($nrvinculom);
                $itemProgOcorrencia->setNrtipoprogocorrencia($progOcorrencia->getNrtipoprogocorrencia());
                $itemProgOcorrencia->setDtinicioprogocor($dtadmissao);
                $itemProgOcorrencia->setIdprogocorconfirmada('S');
                $itemProgOcorrencia->setDsobservacao('Ocorrência padrão da admissão.');
                $itemProgOcorrencia->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $itemProgOcorrencia->setCdoperultatu($this->hcmParameters->getCdoperador());
                $itemProgOcorrencia->setDtinclusao(DateUtil::getDataAtual(false));
                $itemProgOcorrencia->setDtultatu(DateUtil::getDataAtual(false));
                $itemProgOcorrencia->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $itemProgOcorrencia->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                
                $this->entityManager->persist($itemProgOcorrencia);
            }
        } catch (\Exception $e) {
            //$this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function insereBancoHoras($nrorg, $nrtipovinculom, $nrvinculom, $dtadmissao){
        try{
            $arrayBancoHoras = $this->entityManager->getRepository(REPOSITORIES::PTO_BANCOHORASADMI)->findBy(array('nrorg' => $nrorg, 'nrtipovinculom' => $nrtipovinculom));
            foreach($arrayBancoHoras as $bancoHoras){
                $itemBancoHoras = REPOSITORIES::PTO_ALTEBANCOHORAS;
                $itemBancoHoras = new $itemBancoHoras;
                $itemBancoHoras->setNrorg($nrorg);
                $itemBancoHoras->setNraltebancohoras(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_ALTEBANCOHORAS', 1, 12, true));
                $itemBancoHoras->setNrvinculom($nrvinculom);
                $itemBancoHoras->setNrbancohoras($bancoHoras->getNrbancohoras());
                $itemBancoHoras->setDtinicioaltebhoras($dtadmissao);
                $itemBancoHoras->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $itemBancoHoras->setCdoperultatu($this->hcmParameters->getCdoperador());
                $itemBancoHoras->setDtinclusao(DateUtil::getDataAtual(false));
                $itemBancoHoras->setDtultatu(DateUtil::getDataAtual(false));
                $itemBancoHoras->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $itemBancoHoras->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                
                $this->entityManager->persist($itemBancoHoras);
            }
        } catch (\Exception $e) {
            //$this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function insereBeneficios($nrorg, $nrvinculom, $nrpessoa, $idtpbeneficiario, $beneficios){
        try{
            foreach ($beneficios as $beneficio) {
                //INCLUI PARA O DEPENDENTE SE O BENEFICIO TIVER EVENTO MENSALIDADE DEPENDENTE 
                if ($idtpbeneficiario == 'T' || ($idtpbeneficiario == 'D' && isset($beneficio['NREVENTOMENSADEP']))) {
                    $beneficiario = new $this->beneficiario;
                    $beneficiario->setNrorg($nrorg);
                    $beneficiario->setNrvinculom($nrvinculom);
                    $beneficiario->setNrpessoa($nrpessoa);
                    $beneficiario->setIdtpbeneficiario($idtpbeneficiario);
                    $beneficiario->setIdativo($beneficio['IDATIVO']);
                    $beneficiario->setNrbeneficiom(     isset($beneficio['NRBENEFICIOM'])        ? $beneficio['NRBENEFICIOM']       : NULL);
                    $beneficiario->setQtbeneficio(      isset($beneficio['QTBENEFICIO'])         ? $beneficio['QTBENEFICIO']        : NULL);
                    $beneficiario->setNrintegtranspm(   isset($beneficio['NRINTEGTRANSPM'])      ? $beneficio['NRINTEGTRANSPM']     : NULL);
                    $beneficiario->setDsobsbeneficiario(isset($beneficio['DSOBSBENEFICIARIO'])   ? $beneficio['DSOBSBENEFICIARIO']  : NULL);
                    $beneficiario->setNrinscbeneficiario(isset($beneficio['NRINSCBENEFICIARIO']) ? $beneficio['NRINSCBENEFICIARIO'] : NULL);
                    $beneficiario->setDtinscbeneficiario(DateUtil::getDataDeString($beneficio['DTINSCBENEFICIARIO'],DateUtil::FORMATO_BRASILEIRO,true));
                    $beneficiario->setDtcancbeneficiario(isset($beneficio['DTCANCBENEFICIARIO']) ? DateUtil::getDataDeString($beneficio['DTCANCBENEFICIARIO'],DateUtil::FORMATO_BRASILEIRO,true) : null);
                    $beneficiario->setDtinclusao(DateUtil::getDataAtual(false));
                    $beneficiario->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                    $beneficiario->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $beneficiario->setNrbeneficiario(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIARIO', 1, 12, true));
                    $this->entityManager->persist($beneficiario);
                }
            }
        } catch (\Exception $e) {
            //$this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function admissaoVinculo($vinculo, $parametros, $foto, $movimentacao, $pessoa, $endereco, $formaComunicacao, $contaCorrente, $dependente, $solicitacao, $beneficios=null, $valeTransporte = null) {
        $dtmescompetenc = str_split(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($vinculo['DTADMISSAOVINC']))->__toString(), 10);
        $dtmescompetenc = Util::datadeString($dtmescompetenc[0], true);
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        try {
            $this->entityManager->beginTransaction();
            if($pessoa){
                if($pessoa['INSERE_OPERADOR'] === 'S'){
                    $existeOperadorNmacesso = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('nrorg' => $nrorg, 'nmacesso' => $pessoa['EMAIL']));
                    $existeOperadorDsemailoper = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('nrorg' => $nrorg, 'dsemailoper' => $pessoa['EMAIL']));
                    
                    if(!empty($existeOperadorNmacesso)){
                        throw new \Exception(Messages::evaluateMessage(Messages::HCM00000_GER_EMAIL_OPERADOR_CADASTRADO, $existeOperadorNmacesso->getNmoperador()),10);
                    }else if(!empty($existeOperadorDsemailoper)){
                        throw new \Exception(Messages::evaluateMessage(Messages::HCM00000_GER_EMAIL_OPERADOR_CADASTRADO, $existeOperadorDsemailoper->getNmoperador()),10);
                    }

                } 
                $pessoa = $this->verificaIndex($pessoa);
                $pessoa['NMPESSOA'] = Util::removeCaracteresEspeciais($pessoa['NMPESSOA']);
                $pessoa['NMPESSOA'] = strtoupper($pessoa['NMPESSOA']);
                
                //INSERE PARCEIRO NEGOCIO
                $parcnegocio = $this->parcNegocioService->novo($nrorg, $pessoa['NMPESSOA'], $pessoa['NMPESSOA'],'PESSOA','S','N','CPF', $pessoa['NRCPFPESSOA'],Util::datadeString($pessoa['DTNASCPESSOA'],true));

                //INSERE PESSOA
                list($pessoam,$pessoah) = $this->pessoaService->novo($nrorg, $parcnegocio->getNrparcnegocio(), $dtmescompetenc, $pessoa['NMPESSOA'], $pessoa['DSAPELIDOPESS'], 
                         $pessoa['NRCERTNASCPES'], Util::datadeString($pessoa['DTNASCPESSOA'],true),$pessoa['IDSEXOPESSOA'], $pessoa['NRCONDFISPES'], $pessoa['NRCPFPESSOA'], 
                         $pessoa['NRCTPSPESSOA'], $pessoa['NRSERIECTPSPES'], $pessoa['SGUFCTPSPES'], Util::datadeString($pessoa['DTCTPSPESSOA'],true), $pessoa['NRPISPASEPPES'], 
                         Util::datadeString($pessoa['DTINSCPISPASEP'],true), $pessoa['NRINSCINSSPES'], $pessoa['NRINSCISSPES'], $pessoa['CDESTACIVIL'], $pessoa['NRCERTCASAPES'], 
                         Util::datadeString($pessoa['DTCASAPESSOA'],true), $pessoa['IDISENTITUELEI'], $pessoa['NRTITUELEIPES'], $pessoa['NRSECAELEIPES'], $pessoa['NRZONAELEIPES'],
                         $pessoa['NRCARTHABPES'], Util::datadeString($pessoa['DTHABCARTPES'],true), Util::datadeString($pessoa['DTVALHABCARTPES'],true), $pessoa['DSCATEGHABCART'],
                         $pessoa['NRCATMILITARPE'], $pessoa['NRCERTRESEPES'], $pessoa['DSCATEGCERTRESE'], Util::datadeString($pessoa['DTCERTRESEPES'],true), $pessoa['CDEXCERTRESEPES'],
                         $pessoa['NRRACAPESSOA'], $pessoa['IDGRUPSANGUPES'], $pessoa['IDFATORRHPES'], $pessoa['CDPAIS'], $pessoa['SGESTADO'], $pessoa['CDMUNICIPIO'], $pessoa['NRRGPESSOA'],
                         $pessoa['CDEXRGPESSOA'], $pessoa['SGUFRGPESSOA'], $pessoa['DSLOCALEXRGPES'], Util::datadeString($pessoa['DTEXRGPESSOA'],true), $pessoa['NRNACIONALID'], 
                         $pessoa['NRGRAUINSTR'], $pessoa['DSPRONOMETRAT'], $pessoa['NRANOCHEGAPAIS'], $pessoa['NRDOCUESTRANG'], $pessoa['IDPESNATURALIZA'], 
                         Util::datadeString($pessoa['DTNATURALIZAPES'],true), $pessoa['IDPOSVISTOPERM'],  Util::datadeString($pessoa['DTPERMAVISTO'],true), $pessoa['NRCONSELHOREG'], 
                         $pessoa['NRINSCRCONSREG'], Util::datadeString($pessoa['DTEXCONSREG'],true), $pessoa['SGUFCONSREG'], null, $pessoa['NRLIVROCERTNASC'], $pessoa['NRFOLHACERTNASC'], 
                         $pessoa['DSCOMPLCERTNASC'], $pessoa['NRREGCTPSPES'], $pessoa['NRCLASSESTRANG'], $pessoa['DSEXPDOCESTR'],  $pessoa['DTCHEGADAPAIS'], $pessoa['IDCOTAPCD']
                 );
                    
                //INSERE ENDERECO DO PARCEIRO
                if($endereco){
                    $endereco = $this->verificaIndex($endereco);
                    $endereco['CDLOGRADOURO'] = $this->parcNegocioService->convertDsLogrToCdlogr($endereco['DSLOGRADOURO']);
                    $this->parcNegocioService->novoEndereco($parcnegocio->getNrparcnegocio(), $endereco['CDPAIS'], $endereco['SGESTADO'], $endereco['CDMUNICIPIO'], $endereco['CDLOGRADOURO'], $endereco['NMBAIRROENDERECO'],
                    $endereco['CDTIPOENDERECO'], $endereco['NRCEPENDERECO'], $endereco['DSCOMPLEENDERECO'], $endereco['NRIMOVELENDERECO'], $endereco['DSENDERECO'],$endereco['DSREFERENCIAENDE']);
                }
                
                //INSERE CONTA CORRENTE DO PARCEIRO
                if($contaCorrente){
                    $contaCorrente = $this->verificaIndex($contaCorrente);
                    if(!empty($contaCorrente['CDBANCO'])){
                        $this->parcNegocioService->novoContaCorrente($parcnegocio->getNrparcnegocio(), null, $contaCorrente['CDBANCO'],
                        $contaCorrente['CDAGENCIA'], $contaCorrente['CDCONTCORR'], $contaCorrente['NRTPCONTCORR']);
                    }
                }
                
                //INSERE FORMA COMUNICACAO DO PARCEIRO
                if($formaComunicacao){
                    $novaForma = array();
                    foreach ($formaComunicacao as $forma) {
                        $forma['NRPARCNEGOCIO'] = $parcnegocio->getNrparcnegocio();
                        array_push($novaForma,$forma);
                    }
                    $this->parcNegocioService->cadAutoFormaComu($novaForma);
                }
                if($pessoa['INSERE_OPERADOR'] === 'S'){
                    $senha = md5(str_repeat(strtolower(substr($pessoa['NMPESSOA'],0,1)),5));
                    $operador = $this->operadorService->novo($nrorg,$pessoa['NMPESSOA'],Util::datadeString($pessoa['DTNASCPESSOA'],true),$pessoa['EMAIL'],$parcnegocio->getNrparcnegocio(),$senha,'A','A','S','N','003','N','N','N','N','000000000000000000000000000000');
                }
                
            }
            
            //DEFINE NRPESSOA E NRPARCNEGOCIO
            if($pessoa){
                $nrpessoa = $pessoam->getNrpessoa();
                $nrparcnegocio = $parcnegocio->getNrparcnegocio();
            }else{
                $nrpessoa = $vinculo['NRPESSOA'];
                $nrparcnegocio = $vinculo['NRPARCNEGOCIO'];
            }
            //INSERE VINCULO 
            $vinculom = $vinculoh = null;

            function anoData($data) {
                $data = explode("/", $data);
                list($diaAtual, $mesAtual, $anoAtual) = $data;
                return $anoAtual;
            }
            
            $dataAtual = date('d/m/Y');
            $dataAdmissao = $vinculo['DTADMISSAOVINC'];
            
            $anoAtual = anoData($dataAtual);
            $anoAdmissao = anoData($dataAdmissao);
            $anoCompetencia = anoData($this->hcmParameters->getDtCompetencia());
            
            if($vinculo){
                
                if($vinculo['IDMULTVINC'] == 'UNICO'){
                    $vinculoExistente = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculoAtivoPorPessoa($nrorg, $nrpessoa, $dtmescompetenc);
                    if($vinculoExistente){
                        throw new \Exception("Já existe outro vínculo cadastrado para a pessoa. O campo 'Múltiplos Vínculos' não pode ser 'UNICO'.");
                    }
                }
                
                $vinculo = $this->verificaIndex($vinculo);
                $movimentacao = $this->verificaIndex($movimentacao);
                $parametros = $this->verificaIndex($parametros);
                
                $confidencial = 'N';
                
                $vinculo['NMVINCULOM'] = Util::removeCaracteresEspeciais($vinculo['NMVINCULOM']);
                $vinculo['NMVINCULOM'] = strtoupper($vinculo['NMVINCULOM']);
                if(isset($solicitacao['NRTIPOSOLICITACAO'])){
                    $solicitacao["DTMESCOMPETENC"] = $dtmescompetenc;
                }
                
                //Verifica se a organização utiliza aprovação de admissão (ID_APROVACAO_ADMISSAO = 1) para setar histórico do vínculo
                $idAprovAdmissao = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->getIdAprovacaoAdmissao($movimentacao['NRESTRUTLEGAL'], $nrorg);
                if ($anoAtual > $anoAdmissao) {
                    $competencia = $dtmescompetenc;
                    $idAprovAdmissao = 'N';
                } else if ($idAprovAdmissao == 'S' || isset($solicitacao['NRTIPOSOLICITACAO'])) {
                    $competencia =  Util::datadeString("01/01/3000",true);
                } else {
                    $competencia = $dtmescompetenc;
                }
                
                list($vinculom, $vinculoh) = $this->vinculoServices->novo($nrpessoa, Util::datadeString($vinculo['DTADMISSAOVINC'],true), $vinculo['CDMATRICULA'], $vinculo['NRTIPOVINCULOM'], $vinculo['NRTPADMISSAO'],
                     $vinculo['NRVINCULOEMPREG'], Util::datadeString($vinculo['DTOPCAOFGTS'],true), Util::datadeString($vinculo['DTAPOSENTAFGTS'],true), Util::datadeString($vinculo['DTPRIMADMISSAO'],true),$vinculo['NMVINCULOM'],
                     $vinculo['NRDIASCONTRATOEXP'], $vinculo['NRDIASCONTRATOEXPPRO'], Util::datadeString($vinculo['DTEXAMEMEDICO'],true), $competencia, $parametros['NRSITUFUNCM'],
                     $parametros['NROCUPACAOM'], $parametros['NRESCALATRABM'], $vinculo['IDCONTRIBUISIND'], 0, 0, Util::datadeString($vinculo['DTBASEFERIAS'],true),
                     $vinculo['IDREMUNERACAO'], $vinculo['IDTPPAGAMENTO'], $movimentacao['NRESTRUTLEGAL'], $movimentacao['NRESTRUTGEREN'], $parametros['NRTPMODALIDSAL'], $vinculo['IDMULTVINC'],
                     $parametros['NROCUPACAOM'], $vinculo['CDCONTRIBINDIVIDUAL'], $vinculo['NRTPMOVTRANSFM'], Util::datadeString($vinculo['DTFIMCONTRDETERMIN'],true), $movimentacao['NRESTRUTSIND'],
                     $confidencial, $nrorg, null, $vinculo['DSLIVROADMISSAO'], Util::datadeString($vinculo['DTENTREGAEPI'], true), $movimentacao['CDMATRICULAESOCIAL'], $vinculo['DTVENCCONTREXPER'], $vinculo['NRCLAUSASSECURAT'], $vinculo['IDPAGPROPFER13']);
                     
                if ( $anoAtual > $anoAdmissao ) {
                    if ( $anoCompetencia === $anoAtual ) {
                        $this->vinculoServices->novoHistorico(
                            $nrorg,$vinculom->getNrvinculom(), Util::datadeString($this->hcmParameters->getDtCompetencia(),true), $parametros['NRSITUFUNCM'], $vinculoh->getNrcargo(),
                            $vinculoh->getNrescalatrabm(), $vinculoh->getIdcontribuisind(), 0, 0, Util::datadeString($vinculo['DTBASEFERIAS'],true), $vinculoh->getIdremuneracao(),
                            $vinculoh->getIdtppagamento(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrtpmodalidsal(), $vinculo['IDMULTVINC'],
                            $vinculoh->getNrfuncao(), $vinculoh->getCdcontribindividual(), GpeTpmovtransfm::TPMOVTRANSF_ADMIS_EMPR_ANT, null,
                            $vinculoh->getNrestrutsind(), 'N', null, null
                        );
                    } else if ( $anoCompetencia < $anoAtual ) {
                        $dtComp = "01/"."01/".$anoCompetencia;
                        for ( $i = $anoCompetencia; $i <= $anoAtual; $i++ ) {
                            $this->vinculoServices->novoHistorico(
                                $nrorg,$vinculom->getNrvinculom(), Util::datadeString($dtComp,true), $parametros['NRSITUFUNCM'], $vinculoh->getNrcargo(),
                                $vinculoh->getNrescalatrabm(), $vinculoh->getIdcontribuisind(), 0, 0, Util::datadeString($vinculo['DTBASEFERIAS'],true), $vinculoh->getIdremuneracao(),
                                $vinculoh->getIdtppagamento(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrtpmodalidsal(), $vinculo['IDMULTVINC'],
                                $vinculoh->getNrfuncao(), $vinculoh->getCdcontribindividual(), GpeTpmovtransfm::TPMOVTRANSF_ADMIS_EMPR_ANT, null,
                                $vinculoh->getNrestrutsind(), 'N', null, null
                            );
                            $anoCompetencia++;
                            $dtComp = "01/"."01/".$anoCompetencia;
                        }
                    }
                }
                
            }
            //INSERE VALE TRANSPORTE
            if(!empty($valeTransporte)) {
                foreach($valeTransporte as $vale) {
                    $this->valeTransporteService->novo($vale, $vinculom->getNrvinculom());
                }
            }
            
            // INSERE SOLICITAÇÃO
               if(isset($solicitacao['NRTIPOSOLICITACAO'])){
                   $solicitacao['NRVINCULOM'] = $vinculom->getNrvinculom();
                   $this->solicitacaoService->insereSolicitacaoAdm($solicitacao);
               }
            // INSERE MOVIMENTAÇÕES
            if(!empty($movimentacao['NRESTRUTSIND']))
                $this->movimentacaoService->incluiMovimentacao($nrorg,$movimentacao['NRTIPOTRANSFER_SINDICAL'],$vinculom->getNrvinculom(),$movimentacao['NRESTRUTSIND'],Util::datadeString($movimentacao['DTINIMOVIMENT'],true), 10, $movimentacao['NRTPMOVTRANSF']);
            
            if(!empty($movimentacao['NRESTRUTADICIONAL']) && $movimentacao['NRTIPOTRANSFER_ADICIONAL']){
                $this->movimentacaoService->incluiMovimentacao($nrorg,$movimentacao['NRTIPOTRANSFER_ADICIONAL'],$vinculom->getNrvinculom(),$movimentacao['NRESTRUTADICIONAL'],Util::datadeString($movimentacao['DTINIMOVIMENT'],true), $movimentacao['NRTIPOESTRUTURA_ADICIONAL'], $movimentacao['NRTPMOVTRANSF']);
            }
            $this->movimentacaoService->incluiMovimentacao($nrorg,$movimentacao['NRTIPOTRANSFER_LEGAL'],$vinculom->getNrvinculom(), $movimentacao['NRESTRUTLEGAL'],Util::datadeString($movimentacao['DTINIMOVIMENT'],true), $this->hcmParameters->getTipoEstruturaLegalOrg(), $movimentacao['NRTPMOVTRANSF']);
            $this->movimentacaoService->incluiMovimentacao($nrorg,$movimentacao['NRTIPOTRANSFER_GEREN'],$vinculom->getNrvinculom(), $movimentacao['NRESTRUTGEREN'],Util::datadeString($movimentacao['DTINIMOVIMENT'],true), $this->hcmParameters->getTipoEstruturaGerencialOrg(), $movimentacao['NRTPMOVTRANSF']);
            
            // INSERE SITUAÇÃO
            $this->altesitufunc->nova($nrorg,$vinculom->getNrvinculom(),$parametros['NRSITUFUNCM'], Util::datadeString($parametros['DTINISITUFUNC'],true));
            
            // INSERE SALARIO
            $parametros['QTHRLIMITESEPHEXTRA'] = str_replace(':','',$parametros['QTHRLIMITESEPHEXTRA']);
            $this->alteSalario->nova($nrorg,$vinculom->getNrvinculom(),Util::datadeString($parametros['DTALTESALARIO'],true), $parametros['NROCORRENCIA'], $parametros['NRTIPOALTERA'], $this->validaValorMonetario($parametros['VRSALARIO']), $parametros['CDTIPOSALARIO'], $parametros['NRTPMODALIDSAL'],null,$this->validaValorMonetario($parametros['VRDIFERENCAREAL']),$this->validaValorMonetario($parametros['VRPERCREAL']),$this->validaValorMonetario($parametros['VRPERCALTERA']));

            // INSERE ESCALA
            $this->alteEscala->nova($nrorg,$vinculom->getNrvinculom(),$parametros['NRESCALATRABM'], Util::datadeString($parametros['DTINIESCALA'],true),null,Util::datadeString($parametros['DTFIMESCALA'],true), null, null, $parametros['NRTURNO'],$parametros['QTHRLIMITESEPHEXTRA']);
            
            // INSERE OCUPAÇÃO
            $this->alteOcupacao->nova($nrorg,$vinculom->getNrvinculom(),$parametros['NROCUPACAOM'], Util::datadeString($parametros['DTINIOCUPACAO'],true), $parametros['NRTIPOOCUPACAO'], $parametros['NRTPMODALIDSAL'],null,null,Util::datadeString($parametros['DTFIMOCUPACAO'],true));

            //INSERE DEPENDENTES DO PARCEIRO
            if(!empty($dependente)){
                $benefDeps = [];
                foreach ($dependente as $dependentes) {
                    $dependentes = $this->verificaIndex($dependentes);
                    $dependentes['NMPESSOA'] = Util::removeCaracteresEspeciais($dependentes['NMPESSOA']);
                    $dependentes['NMPESSOA'] = strtoupper($dependentes['NMPESSOA']);
                    $dependentes['NRCPFPESSOA'] = str_replace(array('.','/','-'),'',$dependentes['NRCPFPESSOA']);
                    $dtnascdependente = isset($dependentes['DTNASCIFUNDPARC']) ? Util::datadeString($dependentes['DTNASCIFUNDPARC'],true) : null;
                    
                    $addParc = true;
                    if(array_key_exists('NRPARCNEGOCIO', $dependentes) && isset($dependentes['NRPARCNEGOCIO']) && !empty($dependentes['NRPARCNEGOCIO'])) {
                        $depParc = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(array('nrparcnegocio' => $dependentes['NRPARCNEGOCIO'], 'nrorg' => $nrorg));
                        $depPessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array('nrpessoa' => $dependentes['NRPESSOA'], 'nrorg' => $nrorg));
                        if(!empty($depParc)) {
                            $addParc = false;
                            $depParc->setNrorg($nrorg);
                            $depParc->setIdativo('S');
                            $depParc->setNmprincipalparc($dependentes['NMPESSOA']);
                            $depParc->setNmsecundariparc($dependentes['NMPESSOA']);
                            $depParc->setCdtipoinscricao('CPF');
                            $depParc->setNrinscricaoparc($dependentes['NRCPFPESSOA']);
                            $depParc->setDtnascifundparc($dtnascdependente);
                            
                            $depParc->setDtultatu(DateUtil::getDataAtual());
                            $depParc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                            $depParc->setCdoperultatu($this->hcmParameters->getCdoperador());
                            
                            $this->entityManager->persist($depParc);
                            $this->entityManager->flush($depParc); 
                            
                            //INSERE DEPVINCULO SE FOR INFORMADO O TIPO DE DEPENDENTE
                            if(!empty($dependentes['NRTIPODEPENDE'])){
                                foreach ($dependentes['NRTIPODEPENDE'] as $depVinc) {
                                    $this->dependenteVincService->novo($nrorg, $dependentes['NRPESSOA'], $depVinc, $vinculom->getNrvinculom());
                                }
                                //SE FOR DEPENDENTE DE SALARIO FAMILIA, IMPOSTO DE RENDA OU PENSAO INSERE BENFEFICIOS
                                if (in_array(self::TP_SALFAMILIA,$dependentes['NRTIPODEPENDE']) || in_array(self::TP_IRRF,$dependentes['NRTIPODEPENDE']) || in_array(self::TP_PENSAO,$dependentes['NRTIPODEPENDE'])) {
                                    $benefDeps[] = $depPessoa;
                                }
                            }
                        } else {
                            $addParc = true;
                        }
                    }
                    if($addParc) {
                        //INSERE PARCNEGOCIO DO DEPENDENTE
                        $parcnegocioDep = $this->parcNegocioService->novo($nrorg, $dependentes['NMPESSOA'], $dependentes['NMPESSOA'],'PESSOA','S','N','CPF', $dependentes['NRCPFPESSOA'], $dtnascdependente);
                        
                        //INSERE PESSOA DO DEPENDENTE
                        $pessoaDep = $this->pessoaService->novo($nrorg, $parcnegocioDep->getNrparcnegocio(), $dtmescompetenc, $dependentes['NMPESSOA'], null, null, $dtnascdependente,
                             $dependentes['IDSEXOPESSOA'], null, $dependentes['NRCPFPESSOA'], null, null, null, null, null, null,
                             null, null, $dependentes['CDESTACIVIL'], null, null,null, null, null, null, null,
                             null, null, null, null,null, null, null,null, null, null, null, $dependentes['CDPAIS'], $dependentes['SGESTADO'],
                             $dependentes['CDMUNICIPIO'], null, null, null, null, null,$dependentes['NRNACIONALIDADE'], null);
                        
                        //INSERE RELACIONAPARC
                        $this->parcNegocioService->novoRelacionaparc($nrorg, $dependentes['NRTIPORELACIONA'], $nrparcnegocio, Util::datadeString($dependentes['DTFIMRELACIONAMENTO'],true), $parcnegocioDep->getNrparcnegocio());
                        
                        //INSERE DEPVINCULO SE FOR INFORMADO O TIPO DE DEPENDENTE
                        if(!empty($dependentes['NRTIPODEPENDE'])){
                            foreach ($dependentes['NRTIPODEPENDE'] as $depVinc) {
                                $this->dependenteVincService->novo($nrorg, $pessoaDep[0]->getNrpessoa(), $depVinc, $vinculom->getNrvinculom());
                            }
                            //SE FOR DEPENDENTE DE SALARIO FAMILIA, IMPOSTO DE RENDA OU PENSAO INSERE BENFEFICIOS
                            if (in_array(self::TP_SALFAMILIA,$dependentes['NRTIPODEPENDE']) || in_array(self::TP_IRRF,$dependentes['NRTIPODEPENDE']) || in_array(self::TP_PENSAO,$dependentes['NRTIPODEPENDE'])) {
                                $benefDeps[] = $pessoaDep;
                            }
                        }
                    }
                }
            }
            if($foto){
                $foto['IDFOTO'] = 'S';
                $foto['NRPARCNEGOCIO'] = $nrparcnegocio;
                $foto['NRORG'] = $nrorg;
                $foto['DTINCLUSAO'] = DateUtil::getDataAtual();
                $foto['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
                $foto['CDOPERADOR_LOG'] = $this->hcmParameters->getCdoperador();
                $foto['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                $foto['DTULTATU'] = DateUtil::getDataAtual();
                $foto['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                $foto['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                $this->uploadFileService->setGpeAnexos($foto);
            }
            
            //INSERE PROGRAMAÇÃO OCORRÊNCIA DE FREQUÊNCIA
            $this->insereProgOcorrenciaFrequencia($nrorg, $vinculo['NRTIPOVINCULOM'], $vinculom->getNrvinculom(), $vinculom->getDtadmissaovinc());
            
            //INSERE BANCO DE HORAS
            $this->insereBancoHoras($nrorg, $vinculo['NRTIPOVINCULOM'], $vinculom->getNrvinculom(), $vinculom->getDtadmissaovinc());
            
            //INSERE BENEFEICIOS
            if($beneficios){
                $this->insereBeneficios($nrorg, $vinculom->getNrvinculom(), $nrpessoa, 'T', $beneficios);
                foreach ($benefDeps as $benefDep) {
                    $this->insereBeneficios($nrorg, $vinculom->getNrvinculom(), $benefDep[0]->getNrpessoa(), 'D', $beneficios);
                }
            }
            
            $this->entityManager->flush();
            $this->entityManager->commit();
            
            return array('NRVINCULOM'      => $vinculom->getNrvinculom(),
                         'IDAPROVADMISSAO' => $idAprovAdmissao);
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function validaValorMonetario($valor){
        if(!empty($valor)){    
            $valor = str_replace('.','',$valor);
            $valor = str_replace(',','.',$valor);
            return $valor;
        }
        return null;
           
    }
       
}