<?php
namespace HCM\Geral\Services;

use HCM\Geral\Repository\PreFechamentoDctfWebRepository;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class PreFechamentoDctfWebService {
    
    public function __construct(PreFechamentoDctfWebRepository $preFechamentoDctfWebRepository, HCMParameters $hcmParameters) {
        $this->preFechamentoDctfWebRepository = $preFechamentoDctfWebRepository;
        $this->hcmParameters = $hcmParameters;
        $this->contribuicaoPrevidenciariaSegurado = new ContribuicaoPrevidenciariaSegurado($preFechamentoDctfWebRepository);
        $this->informacoesSinteticas = new InformacoesSinteticas($preFechamentoDctfWebRepository);
        $this->informacoesSobreBasesCalculo = new InformacoesSobreBasesCalculo($preFechamentoDctfWebRepository);
    }
    
    public function getContribuicaoPrevidenciaria($parameters) {
        return $this->contribuicaoPrevidenciariaSegurado->getContribuicao($parameters);
    }
    
    public function getEventosApuradosContribuicao($parameters) {
        return $this->contribuicaoPrevidenciariaSegurado->getEventosApurados($parameters);
    }
    
    public function getInformacoesSinteticas($parameters) {
        return $this->informacoesSinteticas->getInformacoesSinteticas($parameters);
    }
    
    public function getInformacoesSobreBasesCalculo($parameters) {
        return $this->informacoesSobreBasesCalculo->getInformacoesSobreBasesCalculo($parameters);
    }
}

class ContribuicaoPrevidenciariaSegurado {
    
    public function __construct(PreFechamentoDctfWebRepository $preFechamentoDctfWebRepository) {
        $this->preFechamentoDctfWebRepository = $preFechamentoDctfWebRepository;
    }
    
    public function getContribuicao($parameters) {
        $results = $this->preFechamentoDctfWebRepository->getContribuicaoPrevidenciariaSegurado($parameters);
        for($i=0; $i<count($results); $i++) {
            $results[$i]['CONCLUSAO'] = $this->getConclusao($results[$i]);
        }
        return $results;
    }
    
    private function getConclusao($params) {
        if( $this->possuiDiferenca($params, 'BASE') ){
            $conclusao = 'Diferença na Base';
        } else if( $this->possuiDiferenca($params, 'SEM_DESCONTO') ){
            $conclusao = 'Sem Desconto de INSS';
        } else if( $this->possuiDiferenca($params, 'DESCONTO') ){
            $conclusao = 'Diferença no Desconto';
        } else if( $this->possuiDiferenca($params, 'ARREDONDAMENTO') ){
            $conclusao = 'Arredondamento';
        } else if( $params['IDSTATUS'] == 'M' ){
            $conclusao = 'Sem retorno do eSocial';
        } else {
            $conclusao = 'Sem Diferença';
        }
        return $conclusao;
    }
    
    private function possuiDiferenca($params, $tipoDiferenca) {
        $base               = $params['BASE'];
        $baseEsocial        = $params['VRBASE_ESO'];
        $desconto           = $params['INSS'];
        $descontoEsocial    = $params['VRCALCULADO_ESO'];
        $diferencaBase      = $params['DIFERENCA_BASE'];
        $diferencaDesconto  = $params['DIFERENCA_DESCONTO'];
        $status             = $params['IDSTATUS'];
   
        switch($tipoDiferenca) {
            case 'BASE':
                $result = ($base != $baseEsocial && $diferencaBase > '.01') && $status != 'M';
                break;
            case 'DESCONTO':
                $result = ($desconto != $descontoEsocial && $diferencaDesconto > '.01') && $status != 'M';
                break;
            case 'ARREDONDAMENTO':
                $result = ($base != $baseEsocial || $desconto != $descontoEsocial) &&
                          ($diferencaBase <= '.01' || $diferencaDesconto <= '.01') && $status != 'M';
                break;
            case 'SEM_DESCONTO':
                $result = ($desconto == null && $descontoEsocial == null || $desconto == 0 && $descontoEsocial == 0) || 
                          ($desconto == 0 && $descontoEsocial == null || $desconto == null && $descontoEsocial == 0) && $status != 'M';
                break;
            default:
                $result = false;
        }
        return $result;
    }
    
    public function getEventosApurados($parameters) {
        $results = $this->preFechamentoDctfWebRepository->getEventosApuradosContribuicao($parameters);
        for($i=0; $i<count($results); $i++){
            if( $results[$i]['VALOR'] == $parameters['DIFERENCA_BASE'] && $parameters['CONCLUSAO'] == 'Diferença na Base' && ($results[$i]['VALOR'] - $parameters['DIFERENCA_BASE']) <= 0.1){
                $results[$i]['OBSERVACAO'] = 'S';
            }
        }
        return $results;
    }
}

class InformacoesSinteticas {
    
    public function __construct(PreFechamentoDctfWebRepository $preFechamentoDctfWebRepository) {
        $this->preFechamentoDctfWebRepository = $preFechamentoDctfWebRepository;
    }
    
    public function getInformacoesSinteticas($parameters) {
        return $this->preFechamentoDctfWebRepository->getInformacoesSinteticas($parameters);
    }
    
}

class InformacoesSobreBasesCalculo {
    
    public function __construct(PreFechamentoDctfWebRepository $preFechamentoDctfWebRepository) {
        $this->preFechamentoDctfWebRepository = $preFechamentoDctfWebRepository;
    }
    
    public function getInformacoesSobreBasesCalculo($parameters) {
        return $this->preFechamentoDctfWebRepository->getInformacoesSobreBasesCalculo($parameters);
    }
    
}