<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use HCM\PTO\Services\MarcacaoPontoService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Messages;
use HCM\Util\Util;

class MarcacaoPontoController extends Crud {

    protected $parameterBagColumns = array('P_NRORG' , 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'NRPERIODOAPURACAO', 'NRVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST', 'P_CONDICAO', 'P_VALOR');
    
    protected const COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER = ['NRVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST'];
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, MarcacaoPontoService $marcacaoPontoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        $this->marcacaoPontoService = $marcacaoPontoService;
        parent::__construct($dataSourceManager);
    }

    public function getMarcacaoPonto (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "getMarcacaoPonto";
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $filterData = Util::createNewFilterColumnConditionsToCommaImploded($filterData, self::COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMarcacaoPontoIntervalo (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "getMarcacaoPontoIntervalo";
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMarcacaoPontoOcorrencias (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "getMarcacaoPontoOcorrencias";
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $filterData = Util::createNewFilterColumnConditionsToCommaImploded($filterData, self::COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setMarcacaoPonto (DTO\Request\DataSet $request, DTO\Response $response) {
        
        try{
            $this->dataSourceName = "GpeMarcacaoponto_setMarcacaoPonto";
            foreach($request->getDataSet()->getRows() as &$row) {
                if(array_key_exists('NRMARCACAOPONTO', $row)){
                    if(empty($row['NRMARCACAOPONTO'])){
                        $row['NRMARCACAOPONTO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_MARCACAOPONTO', 1, 12, true);
                    }
                } else {
                    $row['NRMARCACAOPONTO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_MARCACAOPONTO', 1, 12, true);
                }
                if(array_key_exists('IDATIVO', $row)){
                    if($row['IDATIVO'] === ''){
                    
                    $row['IDATIVO'] = 'S';
                    }
                } else {
                    $row['IDATIVO'] = 'S';
                }
                
                // Trata caso de data e hora da marcação serem salvos em campos separados
                if(!empty($row['DTHRMARCACAOPONTO_DATA'])){
                    $horaMarcacaoPonto              = !empty($row['DTHRMARCACAOPONTO_HORA']) ? $row['DTHRMARCACAOPONTO_HORA'] . ':00' : '00:00:00';
                    $dataHoraMarcacaoPontoString    = $row['DTHRMARCACAOPONTO_DATA'] . ' ' . $horaMarcacaoPonto;
                    $dataHoraMarcacaoPonto          = DateUtil::getDataDeString($dataHoraMarcacaoPontoString, DateUtil::FORMATO_DATAHORA);
                    if (is_null($dataHoraMarcacaoPonto)) {
                        throw new \Exception(Messages::HCM07003_PTO_SAVE_MARC_DATA_HORA_MARCACAO_ILLEGAL);
                    }
                    $row['DTHRMARCACAOPONTO']       = $dataHoraMarcacaoPonto->format(DateUtil::FORMATO_DATAHORA);
                }
                unset($row['DTHRMARCACAOPONTO_HORA']);
                unset($row['DTHRMARCACAOPONTO_DATA']);
                
                // Trata caso de data e hora ajustadas da marcação serem salvos em campos separados
                if(!empty($row['DTHRAJUSTADA_DATA'])){
                    $horaMarcacaoPontoAjust             = !empty($row['DTHRAJUSTADA_HORA']) ? $row['DTHRAJUSTADA_HORA'] . ':00' : '00:00:00';
                    $dataHoraMarcacaoPontoAjustString   = $row['DTHRAJUSTADA_DATA'] . ' ' . $horaMarcacaoPontoAjust;
                    $dataHoraMarcacaoPontoAjust         = DateUtil::getDataDeString($dataHoraMarcacaoPontoAjustString, DateUtil::FORMATO_DATAHORA);
                    $row['DTHRAJUSTADA']                = $dataHoraMarcacaoPontoAjust->format(DateUtil::FORMATO_DATAHORA);
                }
                unset($row['DTHRAJUSTADA_HORA']);
                unset($row['DTHRAJUSTADA_DATA']);
            }
            parent::save($request, $response);
            
            // Atualizar ocorrência de almoço, caso exista?
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "GpeMarcacaoponto_setMarcacaoPonto";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importaArquivoParaMarcacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
            $row['NRORG_PADRAO'] = $this->hcmParameters->getNrorgPadrao();
            $row['MESCOMPETENCIA'] = $this->hcmParameters->getDtCompetencia();
            foreach ($row['DSPATH'] as $arquivo) {
                $row['DSPATH'] = [$arquivo];
                $mensagem = $this->marcacaoPontoService->importaArquivoParaMarcacao($row);    
            }
            if(array_key_exists('erro', $mensagem)){
                $message = new \Zeedhi\Framework\DTO\Response\Message($mensagem['erro'], 'E');
                $response->addMessage($message);
            }
            if($mensagem['sucesso']){
                $message = new \Zeedhi\Framework\DTO\Response\Message($mensagem['sucesso'], 'S');
                $response->addMessage($message);
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setMarcacoesCadastroAutomatico(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow();
            
            $diaInicio  = isset($row['MARC_DAYS']) ? $row['MARC_DAYS']['formattedStart'] : null;
            $diaFim     = isset($row['MARC_DAYS']) ? $row['MARC_DAYS']['formattedEnd'] : null;
            
            if (is_null($diaInicio) || is_null($diaFim)) {
                throw new \Exception(Messages::HCM07003_CADAUTO_UNSUFICCIENT_ITEMS, Messages::ALERT_MESSAGE_CODE);
            }
            
            $nrvinculomList     = $row['NRVINCULOM_LIST'] ?? null;
            $nrestrutlegalList  = $row['NRESTRUTLEGAL_LIST'] ?? null;
            $nrestrutgerenList  = $row['NRESTRUTGEREN_LIST'] ?? null;
            
            $marcacaoList = $row['MARCACAO_LIST'];
            
            $result = $this->marcacaoPontoService->setMarcacoesCadastroAutomatico($diaInicio, $diaFim, $marcacaoList, $nrestrutlegalList, $nrestrutgerenList, $nrvinculomList);
            
            $dataSet = array('success' => $result);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $this->parameterBag->set('NRORG', $this->hcmParameters->getNrorgTrabalho());  
    

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}