<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 21/09/2016
 * Time: 11:14
 */

namespace HCM\SST\Controllers;

use HCM\Util\Messages;
use HCM\SST\Services\ProgEntregaEPIService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ProgEntregaEPIController extends Crud {

    protected $dataSourceName = 'sst_formprogepi';
    protected $dataSourceName_get = 'sst_formprogepi_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERINCLUSAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ProgEntregaEPIService $progEntregaEPIService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->progEntregaEPIService = $progEntregaEPIService;
        parent::__construct($dataSourceManager);
    }

    public function getFormprogepi(DTO\Request\Filter $request, DTO\Response $response) {
        $validaHrsAtend = false;
        $newConditions = array();
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        foreach($filterData->getConditions() as $condition){
            if($condition['columnName'] == 'QTHRDIAENTREPI'){
                $validaHrsAtend = true;
                $condition['value'] = str_replace(":", "", $condition['value']);
            }
            array_push($newConditions, $condition);
        }
        if($validaHrsAtend){
            $filterData = new FilterCriteria(
                                                $filterData->getDataSourceName(),
                                                $newConditions,
                                                $filterData->getPage(),
                                                $filterData->getPageSize()
                                            );
        }
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setFormprogepi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (!array_key_exists('DTLIMITE', $row)) {
                    $row['DTLIMITE'] = null;
                }
                if (array_key_exists('NRFORMPROGEPI', $row)) {
                    if ($row['NRFORMPROGEPI'] === '') {
                        $row['NRFORMPROGEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_FORMPROGEPI', 1, 12, true);
                    }
                } else {
                    $row['NRFORMPROGEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_FORMPROGEPI', 1, 12, true);
                }
                
                if(!isset($row['CDPERIENTR'])){
                    $row['CDPERIENTR'] = null;
                }
            }
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFormprogepi(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getSimulacaoProgEntregaEpi(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $dsordem = null;
            $dsinicio = null;
            $dsfim = null;
            $qtminatendmedio = null;
            $qtatendentrepi = null;
            $qthrdiaentrepi = null;
            $dtinicial = null;
            $dtfinal = null;
            foreach ($request->getFilterCriteria()->getConditions() as $condition) {
                $dsordem = $condition['columnName'] == 'DSORDEM' ? $condition['value'] : $dsordem;
                $dsinicio = $condition['columnName'] == 'DSINICIO' ? $condition['value'] : $dsinicio;
                $dsfim = $condition['columnName'] == 'DSFIM' ? $condition['value'] : $dsfim;
                $qtminatendmedio = $condition['columnName'] == 'QTMINATENDMEDIO' ? $condition['value'] : $qtminatendmedio;
                $qtatendentrepi = $condition['columnName'] == 'QTATENDENTREPI' ? $condition['value'] : $qtatendentrepi;
                $qthrdiaentrepi = $condition['columnName'] == 'QTHRDIAENTREPI' ? $condition['value'] : $qthrdiaentrepi;
                $dtinicial = $condition['columnName'] == 'DTINICIAL' ? $condition['value'] : $dtinicial;
                $dtfinal = $condition['columnName'] == 'DTFINAL' ? $condition['value'] : $dtfinal;
            }
            $dataSet = $this->progEntregaEPIService->getSimulacao($dsordem, $dsinicio, $dsfim, $qtminatendmedio, $qtatendentrepi, $qthrdiaentrepi, $dtinicial, $dtfinal);
            if(empty($dataSet)) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM05000_SST_PROGENTREPI_DADOS_NOT_FOUND_ERROR, DTO\Response\Message::TYPE_ERROR));
            }
            $response->addDataSet(new DataSet('getSimulacaoProgEntregaEpi', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function gravarProgramacaoEntregaEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dsordem = $row['DSORDEM'];
                $dsinicio = $row['DSINICIO'];
                $dsfim = $row['DSFIM'];
                $qtminatendmedio = $row['QTMINATENDMEDIO'];
                $qtatendentrepi = $row['QTATENDENTREPI'];
                $qthrdiaentrepi = $row['QTHRDIAENTREPI'];
                $dtinicial = $row['DTINICIAL'];
                $dtfinal = $row['DTFINAL'];

                $this->progEntregaEPIService->gravar($dsordem, $dsinicio, $dsfim, $qtminatendmedio, $qtatendentrepi, $qthrdiaentrepi, $dtinicial, $dtfinal);
                $response->addNotification(new DTO\Response\Notification('gravarProgramacaoEntregaEpi'), DTO\Response\Notification::TYPE_SUCCESS);
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}